/*
 * Direct3D 10
 *
 * Copyright 2007 Andras Kovacs
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#include "config.h"

#include "d3d10_private.h"

WINE_DEFAULT_DEBUG_CHANNEL(d3d10);

/* ID3D10Asynchronous Implementation */
static HRESULT WINAPI ID3D10AsynchronousImpl_QueryInterface(ID3D10Asynchronous *iface, REFIID riid,
               void **obj) {
    ID3D10AsynchronousImpl *This = (ID3D10AsynchronousImpl *)iface;

    TRACE("(%p)->(%s,%p)\n", This, debugstr_guid(riid), obj);
    if(IsEqualGUID(riid, &IID_IUnknown) ||
       IsEqualGUID(riid, &IID_ID3D10Asynchronous)) {
        IUnknown_AddRef(iface);
        *obj = This;
        return S_OK;
    }

    WARN("(%p)->(%s,%p),not found\n", This, debugstr_guid(riid), obj);
    *obj = NULL;
    return E_NOINTERFACE;
}

static ULONG WINAPI ID3D10AsynchronousImpl_AddRef(ID3D10Asynchronous *iface) {
    ID3D10AsynchronousImpl *This = (ID3D10AsynchronousImpl *)iface;
    ULONG ref = InterlockedIncrement(&This->asynchronous.child.ref);

    TRACE("(%p) : AddRef from %d\n", This, ref - 1);
    return ref;
}

static ULONG WINAPI ID3D10AsynchronousImpl_Release(ID3D10Asynchronous *iface) {
    ID3D10AsynchronousImpl *This = (ID3D10AsynchronousImpl *)iface;
    ULONG ref;

    ref = InterlockedDecrement(&This->asynchronous.child.ref);
    TRACE("(%p) : ReleaseRef to %d\n", This, ref);

    if (ref == 0) {
        HeapFree(GetProcessHeap(), 0, This);
    }

    return ref;
}

void WINAPI ID3D10AsynchronousImpl_Begin(ID3D10Asynchronous *iface) {
    ID3D10AsynchronousImpl *This = (ID3D10AsynchronousImpl *)iface;

    FIXME("(This=%p) stub\n", This);
}

void WINAPI ID3D10AsynchronousImpl_End(ID3D10Asynchronous *iface) {
    ID3D10AsynchronousImpl *This = (ID3D10AsynchronousImpl *)iface;

    FIXME("(This=%p) stub\n", This);
}

HRESULT WINAPI ID3D10AsynchronousImpl_GetData(ID3D10Asynchronous *iface, void *pData, UINT DataSize,
        UINT GetDataFlags) {
    ID3D10AsynchronousImpl *This = (ID3D10AsynchronousImpl *)iface;

    FIXME("(This=%p, pData=%p, DataSize=%d, GetDataFlags=%d) stub\n", This, pData, DataSize, GetDataFlags); /* FIXME */

    return S_OK;
}

UINT WINAPI ID3D10AsynchronousImpl_GetDataSize(ID3D10Asynchronous *iface) {
    ID3D10AsynchronousImpl *This = (ID3D10AsynchronousImpl *)iface;

    FIXME("(This=%p) stub\n", This);

    return 0; /* FIXME */
}

static void WINAPI ID3D10AsynchronousImpl_GetDevice(ID3D10Asynchronous *iface, ID3D10Device **ppDevice) {
    ID3D10DeviceChildImpl_GetDevice((ID3D10DeviceChild *) iface, ppDevice);
}

static HRESULT WINAPI ID3D10AsynchronousImpl_GetPrivateData(ID3D10Asynchronous *iface, REFGUID guid,
                      UINT *pDataSize, void *pData) {
    return ID3D10DeviceChildImpl_GetPrivateData((ID3D10DeviceChild *) iface, guid, pDataSize, pData);
}

static HRESULT WINAPI ID3D10AsynchronousImpl_SetPrivateData(ID3D10Asynchronous *iface, REFGUID guid,
                      UINT DataSize, const void *pData) {
    return ID3D10DeviceChildImpl_SetPrivateData((ID3D10DeviceChild *) iface, guid, DataSize, pData);
}

static HRESULT WINAPI ID3D10AsynchronousImpl_SetPrivateDataInterface(ID3D10Asynchronous *iface, REFGUID guid,
                      const IUnknown *pData) {
    return ID3D10DeviceChildImpl_SetPrivateDataInterface((ID3D10DeviceChild *) iface, guid, pData);
}

const ID3D10AsynchronousVtbl D3D10Asynchronous_Vtbl =
{
    ID3D10AsynchronousImpl_QueryInterface,
    ID3D10AsynchronousImpl_AddRef,
    ID3D10AsynchronousImpl_Release,
    ID3D10AsynchronousImpl_GetDevice,
    ID3D10AsynchronousImpl_GetPrivateData,
    ID3D10AsynchronousImpl_SetPrivateData,
    ID3D10AsynchronousImpl_SetPrivateDataInterface,
    ID3D10AsynchronousImpl_Begin,
    ID3D10AsynchronousImpl_End,
    ID3D10AsynchronousImpl_GetData,
    ID3D10AsynchronousImpl_GetDataSize
};
