/*
 * Direct3D 10
 *
 * Copyright 2007 Andras Kovacs
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#include "config.h"

#include "d3d10_private.h"

WINE_DEFAULT_DEBUG_CHANNEL(d3d10);

/* ID3D10Buffer Implementation */

static HRESULT WINAPI ID3D10BufferImpl_QueryInterface(ID3D10Buffer *iface, REFIID riid, void **obj) {
    ID3D10BufferImpl *This = (ID3D10BufferImpl *)iface;

    TRACE("(%p)->(%s,%p)\n", This, debugstr_guid(riid), obj);
    if(IsEqualGUID(riid, &IID_IUnknown) ||
       IsEqualGUID(riid, &IID_ID3D10Buffer)) {
        IUnknown_AddRef(iface);
        *obj = This;
        return S_OK;
    }

    WARN("(%p)->(%s,%p),not found\n", This, debugstr_guid(riid), obj);
    *obj = NULL;
    return E_NOINTERFACE;
}

static ULONG WINAPI ID3D10BufferImpl_AddRef(ID3D10Buffer *iface) {
    ID3D10BufferImpl *This = (ID3D10BufferImpl *)iface;
    ULONG ref = InterlockedIncrement(&This->resource.child.ref);

    TRACE("(%p) : AddRef from %d\n", This, ref - 1);
    return ref;
}

static ULONG WINAPI ID3D10BufferImpl_Release(ID3D10Buffer *iface) {
    ID3D10BufferImpl *This = (ID3D10BufferImpl *)iface;
    ULONG ref;

    ref = InterlockedDecrement(&This->resource.child.ref);
    TRACE("(%p) : ReleaseRef to %d\n", This, ref);

    if (ref == 0) {
        HeapFree(GetProcessHeap(), 0, This);
    }

    return ref;
}

static void WINAPI ID3D10BufferImpl_GetType(ID3D10Buffer *iface, D3D10_RESOURCE_DIMENSION *rType) {
    ID3D10ResourceImpl_GetType((ID3D10Resource *) iface, rType);
}

static void WINAPI ID3D10BufferImpl_SetEvictionPriority(ID3D10Buffer *iface, UINT EvictionPriority) {
    ID3D10ResourceImpl_SetEvictionPriority((ID3D10Resource *) iface, EvictionPriority);
}

static UINT WINAPI ID3D10BufferImpl_GetEvictionPriority(ID3D10Buffer *iface) {
    return ID3D10ResourceImpl_GetEvictionPriority((ID3D10Resource *) iface);
}

static void WINAPI ID3D10BufferImpl_GetDevice(ID3D10Buffer *iface, ID3D10Device **ppDevice) {
    ID3D10DeviceChildImpl_GetDevice((ID3D10DeviceChild *) iface, ppDevice);
}

static HRESULT WINAPI ID3D10BufferImpl_GetPrivateData(ID3D10Buffer *iface, REFGUID guid,
                      UINT *pDataSize, void *pData) {
    return ID3D10DeviceChildImpl_GetPrivateData((ID3D10DeviceChild *) iface, guid, pDataSize, pData);
}

static HRESULT WINAPI ID3D10BufferImpl_SetPrivateData(ID3D10Buffer *iface, REFGUID guid,
                      UINT DataSize, const void *pData) {
    return ID3D10DeviceChildImpl_SetPrivateData((ID3D10DeviceChild *) iface, guid, DataSize, pData);
}

static HRESULT WINAPI ID3D10BufferImpl_SetPrivateDataInterface(ID3D10Buffer *iface, REFGUID guid,
                      const IUnknown *pData) {
    return ID3D10DeviceChildImpl_SetPrivateDataInterface((ID3D10DeviceChild *) iface, guid, pData);
}

static HRESULT WINAPI ID3D10BufferImpl_Map(ID3D10Buffer *iface, D3D10_MAP MapType,
                      UINT MapFlags, void **ppData) {
    ID3D10BufferImpl *This = (ID3D10BufferImpl *)iface;

    FIXME("(This=%p, MapType=%s, MapFlags=%d, ppData=%p) stub\n",
          This, debug_d3d10_map(MapType), MapFlags, ppData);
                      
    return S_OK;
}

static void WINAPI ID3D10BufferImpl_Unmap(ID3D10Buffer *iface) {
    ID3D10BufferImpl *This = (ID3D10BufferImpl *)iface;

    FIXME("(This=%p) stub\n", This);
}

static void WINAPI ID3D10BufferImpl_GetDesc(ID3D10Buffer *iface, D3D10_BUFFER_DESC *pDesc) {
    ID3D10BufferImpl *This = (ID3D10BufferImpl *)iface;

    FIXME("(This=%p, pDesc=%p) stub\n", This, pDesc);
    dump_d3d10_buffer_desc(pDesc);
}

const ID3D10BufferVtbl D3D10Buffer_Vtbl =
{
    ID3D10BufferImpl_QueryInterface,
    ID3D10BufferImpl_AddRef,
    ID3D10BufferImpl_Release,
    ID3D10BufferImpl_GetDevice,
    ID3D10BufferImpl_GetPrivateData,
    ID3D10BufferImpl_SetPrivateData,
    ID3D10BufferImpl_SetPrivateDataInterface,
    ID3D10BufferImpl_GetType,
    ID3D10BufferImpl_SetEvictionPriority,
    ID3D10BufferImpl_GetEvictionPriority,
    ID3D10BufferImpl_Map,
    ID3D10BufferImpl_Unmap,
    ID3D10BufferImpl_GetDesc
};
