/*
 * Direct3D 10
 *
 * Copyright 2007 Andras Kovacs
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#include "config.h"

#include "d3d10_private.h"

WINE_DEFAULT_DEBUG_CHANNEL(d3d10);

static HRESULT WINAPI ID3D10DeviceChildImpl_QueryInterface(ID3D10DeviceChild *iface, REFIID riid,
                      void **obj) {
    ID3D10DeviceChildImpl *This = (ID3D10DeviceChildImpl *)iface;

    TRACE("(%p)->(%s,%p)\n", This, debugstr_guid(riid), obj);
    if(IsEqualGUID(riid, &IID_IUnknown) ||
       IsEqualGUID(riid, &IID_ID3D10DeviceChild)) {
        IUnknown_AddRef(iface);
        *obj = This;
        return S_OK;
    }

    WARN("(%p)->(%s,%p),not found\n", This, debugstr_guid(riid), obj);
    *obj = NULL;
    return E_NOINTERFACE;
}

static ULONG WINAPI ID3D10DeviceChildImpl_AddRef(ID3D10DeviceChild *iface) {
    ID3D10DeviceChildImpl *This = (ID3D10DeviceChildImpl *)iface;
    ULONG ref = InterlockedIncrement(&This->child.ref);

    TRACE("(%p) : AddRef from %d\n", This, ref - 1);
    return ref;
}

static ULONG WINAPI ID3D10DeviceChildImpl_Release(ID3D10DeviceChild *iface) {
    ID3D10DeviceChildImpl *This = (ID3D10DeviceChildImpl *)iface;
    ULONG ref;

    ref = InterlockedDecrement(&This->child.ref);
    TRACE("(%p) : ReleaseRef to %d\n", This, ref);

    if (ref == 0) {
        HeapFree(GetProcessHeap(), 0, This);
    }

    return ref;
}

void WINAPI ID3D10DeviceChildImpl_GetDevice(ID3D10DeviceChild *iface, ID3D10Device **ppDevice) {
    ID3D10DeviceChildImpl *This = (ID3D10DeviceChildImpl *)iface;

    FIXME("(This=%p, ppDevice=%p) stub\n", This, ppDevice);
}

HRESULT WINAPI ID3D10DeviceChildImpl_GetPrivateData(ID3D10DeviceChild *iface, REFGUID guid,
                      UINT *pDataSize, void *pData) {
    ID3D10DeviceChildImpl *This = (ID3D10DeviceChildImpl *)iface;

    FIXME("(This=%p, guid=%s, pDataSize=%p, pData=%p) stub\n", This, debugstr_guid(guid), pDataSize, pData);

    return S_OK;
}

HRESULT WINAPI ID3D10DeviceChildImpl_SetPrivateData(ID3D10DeviceChild *iface, REFGUID guid,
                      UINT DataSize, const void *pData) {
    ID3D10DeviceChildImpl *This = (ID3D10DeviceChildImpl *)iface;

    FIXME("(This=%p, guid=%s, DataSize=%d, pData=%p) stub\n", This, debugstr_guid(guid), DataSize, pData);

    return S_OK;
}

HRESULT WINAPI ID3D10DeviceChildImpl_SetPrivateDataInterface(ID3D10DeviceChild *iface, REFGUID guid,
                      const IUnknown *pData) {
    ID3D10DeviceChildImpl *This = (ID3D10DeviceChildImpl *)iface;

    FIXME("(This=%p, guid=%s, pData=%p) stub\n", This, debugstr_guid(guid), pData);

    return S_OK;
}


const ID3D10DeviceChildVtbl D3D10DeviceChild_Vtbl =
{
    ID3D10DeviceChildImpl_QueryInterface,
    ID3D10DeviceChildImpl_AddRef,
    ID3D10DeviceChildImpl_Release,
    ID3D10DeviceChildImpl_GetDevice,
    ID3D10DeviceChildImpl_GetPrivateData,
    ID3D10DeviceChildImpl_SetPrivateData,
    ID3D10DeviceChildImpl_SetPrivateDataInterface
};
