/*
 * Direct3D 10
 *
 * Copyright 2007 Andras Kovacs
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#include "config.h"
#include "initguid.h"
#include "d3d10_private.h"


WINE_DEFAULT_DEBUG_CHANNEL(d3d10);

/* At process attach */
BOOL WINAPI DllMain(HINSTANCE hInstDLL, DWORD fdwReason, LPVOID lpv)
{
    TRACE("fdwReason=%d\n", fdwReason);
    switch(fdwReason)
    {
    case DLL_WINE_PREATTACH:
        return FALSE;  /* prefer native version */
    case DLL_PROCESS_ATTACH:
        DisableThreadLibraryCalls( hInstDLL );
        break;
    }
    return TRUE;
}

/* FIXME: Define the interface for the Adapter parameter */
HRESULT WINAPI D3D10CreateDevice(void *Adapter, D3D10_DRIVER_TYPE DriverType, HMODULE Software, UINT Flags,
                                 UINT SDKVersion, ID3D10Device **Device) {
    ID3D10DeviceImpl *object;

    TRACE("Adapter %p, Driver Type %d, Software Module handle %p, Flags %08x, SDKVersion %d, Device %p\n",
          Adapter, DriverType, Software, Flags, SDKVersion, Device);

    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));

    if(!object) {
        ERR("Out of memory when allocating a D3D10 Device implementation structure\n");
        *Device = NULL;
        return E_OUTOFMEMORY;
     }

    object->lpVtbl = &D3D10Device_Vtbl;
    object->ref = 1;
    *Device = (ID3D10Device *) object;

    TRACE("Created device at %p\n", object);

    return S_OK;
}
