/*
 * Direct3D 10 private include file
 *
 * Copyright 2007 Andras Kovacs 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */

#ifndef __WINE_D3D10_PRIVATE_H
#define __WINE_D3D10_PRIVATE_H

#include <stdarg.h>

#define NONAMELESSUNION
#define NONAMELESSSTRUCT
#define COBJMACROS
#include "windef.h"
#include "winbase.h"
#include "wingdi.h"
#include "winuser.h"
#include "wine/debug.h"
#include "wine/unicode.h"

#include "objbase.h"
#include "d3d10.h"
#include "d3d10_misc.h"

#include "wine/wined3d_interface.h"

/**************************************************************************
* ID3D10Device implementation structure
*/
typedef struct ID3D10DeviceImpl
{
    /* IUnknown fields */
    const ID3D10DeviceVtbl      *lpVtbl;
    LONG                        ref;
} ID3D10DeviceImpl;

extern const ID3D10DeviceVtbl D3D10Device_Vtbl;

/**************************************************************************
* ID3D10DeviceChild implementation structure
*/

typedef struct DeviceChildClass
{
    LONG                             ref;
    /* Further device child implementation members go here */
    ID3D10DeviceImpl                 device;
    /* struct list privateDataList */
    /* ... */
} DeviceChildClass;

typedef struct ID3D10DeviceChildImpl
{
    /* IUnknown fields */
    const ID3D10DeviceChildVtbl      *lpVtbl;
    DeviceChildClass                 child;
    /* No more members here. They should go into DeviceChildClass */
} ID3D10DeviceChildImpl;

extern const ID3D10DeviceChildVtbl D3D10DeviceChild_Vtbl;

/**************************************************************************
* ID3D10Resource implementation structure
*/
typedef struct ResourceClass
{
    DeviceChildClass                 child;
    int                              priority;
    /* Further resource stuff, that is too specific for device childs goes here */
} ResourceClass;

typedef struct ID3D10ResourceImpl
{
    const ID3D10ResourceVtbl      *lpVtbl;
    ResourceClass                 resource;
    /* No more members here. They should go into ResourceClass */
} ID3D10ResourceImpl;

extern const ID3D10ResourceVtbl D3D10Resource_Vtbl;

/**************************************************************************
* ID3D10Texture1D implementation structure
*/
typedef struct ID3D10Texture1DImpl
{
    const ID3D10Texture1DVtbl     *lpVtbl;
    ResourceClass                 resource;
    /* ID3D10Texture1DImpl specific stuff goes here */
} ID3D10Texture1DImpl;

extern const ID3D10Texture1DVtbl D3D10Texture1D_Vtbl;

/**************************************************************************
* ID3D10Buffer implementation structure
*/
typedef struct ID3D10BufferImpl
{
    const ID3D10BufferVtbl        *lpVtbl;
    ResourceClass                 resource;
    /* ID3D10BufferImpl specific stuff goes here */
} ID3D10BufferImpl;

extern const ID3D10BufferVtbl D3D10Buffer_Vtbl;

/**************************************************************************
* ID3D10Texture2D implementation structure
*/
typedef struct ID3D10Texture2DImpl
{
    const ID3D10Texture2DVtbl     *lpVtbl;
    ResourceClass                 resource;
} ID3D10Texture2DImpl;

extern const ID3D10Texture2DVtbl D3D10Texture2D_Vtbl;

/**************************************************************************
* ID3D10Texture3D implementation structure
*/
typedef struct ID3D10Texture3DImpl
{
    const ID3D10Texture3DVtbl     *lpVtbl;
    ResourceClass                 resource;
} ID3D10Texture3DImpl;

extern const ID3D10Texture3DVtbl D3D10Texture3D_Vtbl;

/**************************************************************************
* ID3D10BlendState implementation structure
*/
typedef struct ID3D10BlendStateImpl
{
    const ID3D10BlendStateVtbl    *lpVtbl;
    DeviceChildClass              child;
} ID3D10BlendStateImpl;

extern const ID3D10BlendStateVtbl D3D10BlendState_Vtbl;

/**************************************************************************
* ID3D10DepthStencilState implementation structure
*/
typedef struct ID3D10DepthStencilImpl
{
    const ID3D10DepthStencilStateVtbl    *lpVtbl;
    DeviceChildClass                 child;
    /* Depth stencil state specific stuff goes here */
} ID3D10DepthStencilStateImpl;

extern const ID3D10DepthStencilStateVtbl D3D10DepthStencilState_Vtbl;

/**************************************************************************
* ID3D10View implementation structure
*/
typedef struct ViewClass {
    DeviceChildClass                 child;
} ViewClass;

typedef struct ID3D10ViewImpl
{
    const ID3D10ViewVtbl         *lpVtbl;
    ViewClass                    view;
    /* Nothing more here */
} ID3D10ViewImpl;

extern const ID3D10ViewVtbl D3D10View_Vtbl;

/**************************************************************************
* ID3D10DepthStencilView implementation structure
*/
typedef struct ID3D10DepthStencilViewImpl
{
    const ID3D10DepthStencilViewVtbl         *lpVtbl;
    ViewClass                               view;
    /* Depth stencil view specific stuff goes here */
} ID3D10DepthStencilViewImpl;

extern const ID3D10DepthStencilViewVtbl D3D10DepthStencilView_Vtbl;

/**************************************************************************
* ID3D10RenderTargetView implementation structure
*/
typedef struct ID3D10RenderTargetViewImpl
{
    const ID3D10RenderTargetViewVtbl         *lpVtbl;
    ViewClass                    view;
    /* Render target view specific stuff goes here */
} ID3D10RenderTargetViewImpl;

extern const ID3D10RenderTargetViewVtbl D3D10RenderTargetView_Vtbl;

/**************************************************************************
* ID3D10ShaderResourceView implementation structure
*/
typedef struct ID3D10ShaderResourceViewImpl
{
    const ID3D10ShaderResourceViewVtbl       *lpVtbl;
    DeviceChildClass       		     child;
} ID3D10ShaderResourceViewImpl;

extern const ID3D10ShaderResourceViewVtbl D3D10ShaderResourceView_Vtbl;

/**************************************************************************
* ID3D10GeometryShader implementation structure
*/
typedef struct ID3D10GeometryShaderImpl
{
    const ID3D10GeometryShaderVtbl       *lpVtbl;
    DeviceChildClass        		 child;
} ID3D10GeometryShaderImpl;

extern const ID3D10GeometryShaderVtbl D3D10GeometryShader_Vtbl;

/**************************************************************************
* ID3D10InputLayout implementation structure
*/
typedef struct ID3D10InputLayoutImpl
{
    const ID3D10InputLayoutVtbl          *lpVtbl;
    DeviceChildClass    		 child;
} ID3D10InputLayoutImpl;

extern const ID3D10InputLayoutVtbl D3D10InputLayout_Vtbl;

/**************************************************************************
* ID3D10PixelShader implementation structure
*/
typedef struct ID3D10PixelShaderImpl
{
    const ID3D10PixelShaderVtbl          *lpVtbl;
    DeviceChildClass       		 child;
} ID3D10PixelShaderImpl;

extern const ID3D10PixelShaderVtbl D3D10PixelShader_Vtbl;

/**************************************************************************
* ID3D10RasterizerState implementation structure
*/
typedef struct ID3D10RasterizerStateImpl
{
    const ID3D10RasterizerStateVtbl      *lpVtbl;
    DeviceChildClass       		 child;
} ID3D10RasterizerStateImpl;

extern const ID3D10RasterizerStateVtbl D3D10RasterizerState_Vtbl;

/**************************************************************************
* ID3D10SamplerState implementation structure
*/
typedef struct ID3D10SamplerStateImpl
{
    const ID3D10SamplerStateVtbl         *lpVtbl;
    DeviceChildClass       		 child;
} ID3D10SamplerStateImpl;

extern const ID3D10SamplerStateVtbl D3D10SamplerState_Vtbl;

/**************************************************************************
* ID3D10VertexShader implementation structure
*/
typedef struct ID3D10VertexShaderImpl
{
    const ID3D10VertexShaderVtbl         *lpVtbl;
    DeviceChildClass       		 child;
} ID3D10VertexShaderImpl;

extern const ID3D10VertexShaderVtbl D3D10VertexShader_Vtbl;

/**************************************************************************
* ID3D10Asynchronous implementation structure
*/

typedef struct AsynchronousClass {
    DeviceChildClass                 child;
} AsynchronousClass;

typedef struct ID3D10AsynchronousImpl
{
    const ID3D10AsynchronousVtbl         *lpVtbl;
    AsynchronousClass			 asynchronous;
} ID3D10AsynchronousImpl;

extern const ID3D10AsynchronousVtbl D3D10Asynchronous_Vtbl;

/**************************************************************************
* ID3D10Counter implementation structure
*/
typedef struct ID3D10CounterImpl
{
    const ID3D10CounterVtbl              *lpVtbl;
    AsynchronousClass			 asynchronous;
} ID3D10CounterImpl;

extern const ID3D10CounterVtbl D3D10Counter_Vtbl;

/**************************************************************************
* ID3D10Query implementation structure
*/

typedef struct QueryClass {
    AsynchronousClass                 asynchronous;
} QueryClass;

typedef struct ID3D10QueryImpl
{
    const ID3D10QueryVtbl                *lpVtbl;
    QueryClass				 query;
} ID3D10QueryImpl;

extern const ID3D10QueryVtbl D3D10Query_Vtbl;

/**************************************************************************
* ID3D10Predicate implementation structure
*/
typedef struct ID3D10PredicateImpl
{
    const ID3D10PredicateVtbl                *lpVtbl;
    QueryClass				     query;
} ID3D10PredicateImpl;

extern const ID3D10PredicateVtbl D3D10Predicate_Vtbl;

/**************************************************************************
* ID3D10Multithread implementation structure
*/
typedef struct ID3D10MultithreadImpl
{
    /* IUnknown fields */
    const ID3D10MultithreadVtbl      *lpVtbl;
    LONG                             ref;
} ID3D10MultithreadImpl;

extern const ID3D10MultithreadVtbl D3D10Multithread_Vtbl;

/* Extern functions */

/* ID3D10DeviceChild */
extern void WINAPI ID3D10DeviceChildImpl_GetDevice(ID3D10DeviceChild *iface, ID3D10Device **ppDevice);
extern HRESULT WINAPI ID3D10DeviceChildImpl_GetPrivateData(ID3D10DeviceChild *iface, REFGUID guid,
                      UINT *pDataSize, void *pData);
extern HRESULT WINAPI ID3D10DeviceChildImpl_SetPrivateData(ID3D10DeviceChild *iface, REFGUID guid,
                      UINT DataSize, const void *pData);
extern HRESULT WINAPI ID3D10DeviceChildImpl_SetPrivateDataInterface(ID3D10DeviceChild *iface, REFGUID guid,
                      const IUnknown *pData);

/* ID3D10Resource */
extern void WINAPI ID3D10ResourceImpl_GetType(ID3D10Resource *iface, D3D10_RESOURCE_DIMENSION *rType);
extern void WINAPI ID3D10ResourceImpl_SetEvictionPriority(ID3D10Resource *iface, UINT EvictionPriority);
extern UINT WINAPI ID3D10ResourceImpl_GetEvictionPriority(ID3D10Resource *iface);

/* ID3D10View */
extern void WINAPI ID3D10ViewImpl_GetResource(ID3D10View *iface, ID3D10Resource **ppResource);

/* ID3D10Asynchronous */
void WINAPI ID3D10AsynchronousImpl_Begin(ID3D10Asynchronous *iface);
void WINAPI ID3D10AsynchronousImpl_End(ID3D10Asynchronous *iface);
HRESULT WINAPI ID3D10AsynchronousImpl_GetData(ID3D10Asynchronous *iface, void *pData, UINT DataSize,
        UINT GetDataFlags);
UINT WINAPI ID3D10AsynchronousImpl_GetDataSize(ID3D10Asynchronous *iface);

/* ID3D10Query */
void WINAPI ID3D10QueryImpl_GetDesc(ID3D10Query *iface, D3D10_QUERY_DESC *pDesc);

/* Debug Functions */
extern void dump_d3d10_blend_desc(const D3D10_BLEND_DESC *pBlendStateDesc);
extern const char *debug_d3d10_map(D3D10_MAP MapType);
extern void dump_d3d10_buffer_desc(D3D10_BUFFER_DESC *pDesc);
extern void dump_d3d10_counter_desc (D3D10_COUNTER_DESC *pDesc);
extern void dump_d3d10_depth_stencil_desc(const D3D10_DEPTH_STENCIL_DESC *pDepthStencilDesc);
extern const char *debug_dxgi_format(const DXGI_FORMAT Format);
/* Debug Functions End */

/* Extern Functions End */

#endif /* __WINE_D3D10_PRIVATE_H */
