/*
 * Direct3D 10
 *
 * Copyright 2007 Andras Kovacs
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#include "config.h"

#include "d3d10_private.h"

WINE_DEFAULT_DEBUG_CHANNEL(d3d10);

static HRESULT WINAPI ID3D10DepthStencilStateImpl_QueryInterface(ID3D10DepthStencilState *iface, REFIID riid, void **obj) {
    ID3D10DepthStencilStateImpl *This = (ID3D10DepthStencilStateImpl *)iface;

    TRACE("(%p)->(%s,%p)\n", This, debugstr_guid(riid), obj);
    if(IsEqualGUID(riid, &IID_IUnknown) ||
       IsEqualGUID(riid, &IID_ID3D10DepthStencilState)) {
        IUnknown_AddRef(iface);
        *obj = This;
        return S_OK;
    }

    WARN("(%p)->(%s,%p),not found\n", This, debugstr_guid(riid), obj);
    *obj = NULL;
    return E_NOINTERFACE;
}

static ULONG WINAPI ID3D10DepthStencilStateImpl_AddRef(ID3D10DepthStencilState *iface) {
    ID3D10DepthStencilStateImpl *This = (ID3D10DepthStencilStateImpl *)iface;
    ULONG ref = InterlockedIncrement(&This->child.ref);

    TRACE("(%p) : AddRef from %d\n", This, ref - 1);
    return ref;
}

static ULONG WINAPI ID3D10DepthStencilStateImpl_Release(ID3D10DepthStencilState *iface) {
    ID3D10DepthStencilStateImpl *This = (ID3D10DepthStencilStateImpl *)iface;
    ULONG ref;

    ref = InterlockedDecrement(&This->child.ref);
    TRACE("(%p) : ReleaseRef to %d\n", This, ref);

    if (ref == 0) {
        HeapFree(GetProcessHeap(), 0, This);
    }

    return ref;
}

static void WINAPI ID3D10DepthStencilStateImpl_GetDevice(ID3D10DepthStencilState *iface, ID3D10Device **ppDevice) {
    ID3D10DeviceChildImpl_GetDevice((ID3D10DeviceChild *) iface, ppDevice);
}

static HRESULT WINAPI ID3D10DepthStencilStateImpl_GetPrivateData(ID3D10DepthStencilState *iface, REFGUID guid,
                      UINT *pDataSize, void *pData) {
    return ID3D10DeviceChildImpl_GetPrivateData((ID3D10DeviceChild *) iface, guid, pDataSize, pData);
}

static HRESULT WINAPI ID3D10DepthStencilStateImpl_SetPrivateData(ID3D10DepthStencilState *iface, REFGUID guid,
                      UINT DataSize, const void *pData) {
    return ID3D10DeviceChildImpl_SetPrivateData((ID3D10DeviceChild *) iface, guid, DataSize, pData);
}

static HRESULT WINAPI ID3D10DepthStencilStateImpl_SetPrivateDataInterface(ID3D10DepthStencilState *iface, REFGUID guid,
                      const IUnknown *pData) {
    return ID3D10DeviceChildImpl_SetPrivateDataInterface((ID3D10DeviceChild *) iface, guid, pData);
}

static void WINAPI ID3D10DepthStencilStateImpl_GetDesc(ID3D10DepthStencilState *iface, D3D10_DEPTH_STENCIL_DESC *pDesc) {
    ID3D10DepthStencilStateImpl *This = (ID3D10DepthStencilStateImpl *)iface;
    
    FIXME("(This=%p, pDesc=%p) stub\n", This, pDesc);
    dump_d3d10_depth_stencil_desc(pDesc);
}


const ID3D10DepthStencilStateVtbl D3D10DepthStencilState_Vtbl =
{
    ID3D10DepthStencilStateImpl_QueryInterface,
    ID3D10DepthStencilStateImpl_AddRef,
    ID3D10DepthStencilStateImpl_Release,
    ID3D10DepthStencilStateImpl_GetDevice,
    ID3D10DepthStencilStateImpl_GetPrivateData,
    ID3D10DepthStencilStateImpl_SetPrivateData,
    ID3D10DepthStencilStateImpl_SetPrivateDataInterface,
    ID3D10DepthStencilStateImpl_GetDesc
};
