/*
 * Direct3D 10
 *
 * Copyright 2007 Andras Kovacs
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#include "config.h"

#include "d3d10_private.h"

WINE_DEFAULT_DEBUG_CHANNEL(d3d10);

static HRESULT WINAPI ID3D10DeviceImpl_QueryInterface(ID3D10Device *iface, REFIID riid, void **obj) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    TRACE("(%p)->(%s,%p)\n", This, debugstr_guid(riid), obj);
    if(IsEqualGUID(riid, &IID_IUnknown) ||
       IsEqualGUID(riid, &IID_ID3D10Device)) {
        IUnknown_AddRef(iface);
        *obj = This;
        return S_OK;
    }

    WARN("(%p)->(%s,%p),not found\n", This, debugstr_guid(riid), obj);
    *obj = NULL;
    return E_NOINTERFACE;
}

static ULONG WINAPI ID3D10DeviceImpl_AddRef(ID3D10Device *iface) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ULONG ref = InterlockedIncrement(&This->ref);

    TRACE("(%p) : AddRef from %d\n", This, ref - 1);
    return ref;
}

static ULONG WINAPI ID3D10DeviceImpl_Release(ID3D10Device *iface) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ULONG ref;

    ref = InterlockedDecrement(&This->ref);
    TRACE("(%p) : ReleaseRef to %d\n", This, ref);

    if (ref == 0){
        HeapFree(GetProcessHeap(), 0, This);
    }

    return ref;
}

static void WINAPI ID3D10DeviceImpl_VSSetConstantBuffers(ID3D10Device *iface, UINT StartSlot,
                   UINT NumBuffers, ID3D10Buffer **ppConstantBuffers) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumBuffers=%d, ppConstantBuffers=%p) stub\n", This, StartSlot, NumBuffers,
          ppConstantBuffers);
}

static void WINAPI ID3D10DeviceImpl_PSSetShaderResources(ID3D10Device *iface, UINT StartSlot,
                   UINT NumViews, ID3D10ShaderResourceView **ppShaderResourceViews) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumViews=%d, ppShaderResourceViews=%p) stub\n", This, StartSlot,
          NumViews, ppShaderResourceViews);
}

static void WINAPI ID3D10DeviceImpl_PSSetShader(ID3D10Device *iface, ID3D10PixelShader *pPixelShader) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pPixelShader=%p) stub\n", This, pPixelShader);
}

static void WINAPI ID3D10DeviceImpl_PSSetSamplers(ID3D10Device *iface, UINT StartSlot, UINT NumSamplers,
                   ID3D10SamplerState **ppSamplers) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumSamplers=%d, ppSamplerState=%p) stub\n", This, StartSlot, NumSamplers,
          ppSamplers);
}
static void WINAPI ID3D10DeviceImpl_VSSetShader(ID3D10Device *iface, ID3D10VertexShader *pVertexShader) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pVertexShader=%p) stub\n", This, pVertexShader);
}

static void WINAPI ID3D10DeviceImpl_DrawIndexed(ID3D10Device *iface, UINT IndexCount,
                   UINT StartIndexLocation, INT BaseVertexLocation) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, IndexCount=%d, StartIndexLocation=%d, BaseVertexLocation=%d) stub\n", This, IndexCount,
          StartIndexLocation, BaseVertexLocation);
}

static void WINAPI ID3D10DeviceImpl_Draw(ID3D10Device *iface, UINT VertexCount, UINT StartVertexLocation) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, VertexCount=%d, StartVertexLocation=%d) stub\n", This, VertexCount,
          StartVertexLocation);
}

static void WINAPI ID3D10DeviceImpl_PSSetConstantBuffers(ID3D10Device *iface, UINT StartSlot,
                   UINT NumBuffers, ID3D10Buffer **ppConstantBuffers) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumBuffers=%d, ppConstantBuffers=%p) stub\n", This, StartSlot,
          NumBuffers, ppConstantBuffers);
}

static void WINAPI ID3D10DeviceImpl_IASetInputLayout(ID3D10Device *iface, ID3D10InputLayout *pInputLayout) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pInputLayout=%p) stub\n", This, pInputLayout);
}

static void WINAPI ID3D10DeviceImpl_IASetVertexBuffers(ID3D10Device *iface, UINT StartSlot, UINT NumBuffers,
                   ID3D10Buffer **ppVertexBuffers, const UINT *pStrides,
                   const UINT *pOffsets) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumBuffers=%d, ppVertexBuffers=%p, pStrides=%p, pOffsets=%p) stub\n",
          This, StartSlot, NumBuffers, ppVertexBuffers, pStrides, pOffsets);
}

static void WINAPI ID3D10DeviceImpl_IASetIndexBuffer(ID3D10Device *iface, ID3D10Buffer *pIndexBuffer,
                   DXGI_FORMAT Format, UINT Offset) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pIndexBuffer=%p, Format=%d (%s), Offset=%d) stub\n", This, pIndexBuffer,
          (UINT) Format, debug_dxgi_format(Format), Offset);
}

static void WINAPI ID3D10DeviceImpl_DrawIndexedInstanced(ID3D10Device *iface, UINT IndexCountPerInstance,
                   UINT InstanceCount, UINT StartIndexLocation, INT BaseVertexLocation,
                   UINT StartInstanceLocation) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, IndexCountPerInstance=%d, InstanceCount=%d, StartIndexLocation=%d, BaseVertexLocation=%d, StartInstanceLocation=%d) stub\n",
          This, IndexCountPerInstance, InstanceCount, StartIndexLocation, BaseVertexLocation, StartInstanceLocation);
}

static void WINAPI ID3D10DeviceImpl_DrawInstanced(ID3D10Device *iface, UINT VertexCountPerInstance,
                   UINT InstanceCount, UINT StartVertexLocation, UINT StartInstanceLocation) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, VertexCountPerInstance=%d, InstanceCount=%d, StartVertexLocation=%d, StartInstanceLocation=%d) stub\n",
          This, VertexCountPerInstance, InstanceCount, StartVertexLocation, StartInstanceLocation);
}

static void WINAPI ID3D10DeviceImpl_GSSetConstantBuffers(ID3D10Device *iface, UINT StartSlot,
                   UINT NumBuffers, ID3D10Buffer **ppConstantBuffers) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumBuffers=%d, ppConstantBuffers=%p) stub\n", This, StartSlot,
          NumBuffers, ppConstantBuffers);
}

static void WINAPI ID3D10DeviceImpl_GSSetShader(ID3D10Device *iface, ID3D10GeometryShader *pShader) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pShader=%p) stub\n", This, pShader);
}

static void WINAPI ID3D10DeviceImpl_IASetPrimitiveTopology(ID3D10Device *iface,
                   D3D10_PRIMITIVE_TOPOLOGY Topology) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, Topology=%d) stub\n", This, (UINT) Topology); /* FIXME */
}

static void WINAPI ID3D10DeviceImpl_VSSetShaderResources(ID3D10Device *iface, UINT StartSlot,
                   UINT NumViews, ID3D10ShaderResourceView **ppShaderResourceViews) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumViews=%d, ppShaderResourceViews=%p) stub\n",
          This, StartSlot, NumViews, ppShaderResourceViews);
}

static void WINAPI ID3D10DeviceImpl_VSSetSamplers(ID3D10Device *iface, UINT StartSlot, UINT NumSamplers,
                   ID3D10SamplerState **ppSamplers) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumSamplers=%d, ppSamplers=%p) stub\n", This, StartSlot, NumSamplers,
          ppSamplers);
}

static void WINAPI ID3D10DeviceImpl_SetPredication(ID3D10Device *iface, ID3D10Predicate *pPredicate,
                   BOOL PredicateValue) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pPredicate=%p, PredicateValue=%d) stub\n", This, pPredicate, PredicateValue);
}

static void WINAPI ID3D10DeviceImpl_GSSetShaderResources(ID3D10Device *iface, UINT StartSlot, UINT NumViews,
                   ID3D10ShaderResourceView **ppShaderResourceViews) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumViews=%d, ppShaderResourceViews=%p) stub\n",
          This, StartSlot, NumViews, ppShaderResourceViews);
}

static void WINAPI ID3D10DeviceImpl_GSSetSamplers(ID3D10Device *iface, UINT StartSlot, UINT NumSamplers,
                   ID3D10SamplerState **ppSamplers) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumSamplers=%d, ppSamplers=%p) stub\n", This, StartSlot, NumSamplers,
          ppSamplers);
}

static void WINAPI ID3D10DeviceImpl_OMSetRenderTargets(ID3D10Device *iface, UINT NumViews,
                   ID3D10RenderTargetView **ppRenderTargetViews, ID3D10DepthStencilView *pDepthStencilView) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, NumViews=%d, ppRenderTargetViews=%p, pDepthStencilView=%p) stub\n",
          This, NumViews, ppRenderTargetViews, pDepthStencilView);
}

static void WINAPI ID3D10DeviceImpl_OMSetBlendState(ID3D10Device *iface, ID3D10BlendState *pBlendState,
                   const FLOAT BlendFactor[4], UINT SampleMask) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pBlendState=%p, BlendFactor='%f,%f,%f,%f', SampleMask=%d) stub\n", pBlendState,
          This, BlendFactor[0], BlendFactor[1], BlendFactor[2], BlendFactor[3], SampleMask);
}

static void WINAPI ID3D10DeviceImpl_OMSetDepthStencilState(ID3D10Device *iface,
                   ID3D10DepthStencilState *pDepthStencilState, UINT StencilRef) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pDepthStencilState=%p, StencilRef=%d) stub\n", This, pDepthStencilState, StencilRef);
}

static void WINAPI ID3D10DeviceImpl_SOSetTargets(ID3D10Device *iface, UINT NumBuffers,
                   ID3D10Buffer **ppSOTargets, const UINT *pOffsets) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, NumBuffers=%d, ppSOTargets=%p, pOffsets=%p) stub\n", This, NumBuffers,
          ppSOTargets, pOffsets);
}

static void WINAPI ID3D10DeviceImpl_DrawAuto(ID3D10Device *iface) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p) stub\n", This);
}

static void WINAPI ID3D10DeviceImpl_RSSetState(ID3D10Device *iface,
                   ID3D10RasterizerState *pRasterizerState) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pRasterizerState=%p) stub\n", This, pRasterizerState);
}

static void WINAPI ID3D10DeviceImpl_RSSetViewports(ID3D10Device *iface, UINT NumViewports,
                   const D3D10_VIEWPORT *pViewports) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, NumViewports=%d, pViewports=%p) stub\n", This, NumViewports, pViewports); /* FIXME */
}

static void WINAPI ID3D10DeviceImpl_RSSetScissorRects(ID3D10Device *iface, UINT NumRects,
                   const D3D10_RECT *pRects) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, NumRects=%d, pRects=%p) stub\n", This, NumRects, pRects);
}

static void WINAPI ID3D10DeviceImpl_CopySubresourceRegion(ID3D10Device *iface, ID3D10Resource *pDstResource,
                   UINT DstSubresource, UINT DstX, UINT DstY, UINT DstZ, ID3D10Resource *pSrcResource,
                   UINT SrcSubresource, const D3D10_BOX *pSrcBox) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pDstResource=%p, DstSubresource=%d, DstX=%d, DstY=%d, DstZ=%d, pSrcResource=%p, SrcSubresource=%d, pSrcBox=%p) stub\n",
          This, pDstResource, DstSubresource, DstX, DstY, DstZ, pSrcResource, SrcSubresource, pSrcBox);
}

static void WINAPI ID3D10DeviceImpl_CopyResource(ID3D10Device *iface, ID3D10Resource *pDstResource,
                   ID3D10Resource *pSrcResource) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pDstResource=%p, pSrcResource=%p) stub\n", This, pDstResource, pSrcResource);
}

static void WINAPI ID3D10DeviceImpl_UpdateSubresource(ID3D10Device *iface, ID3D10Resource *pDstResource,
                   UINT DstSubresource, const D3D10_BOX *pDstBox, const void *pSrcData, UINT SrcRowPitch,
                   UINT SrcDepthPitch) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pDstResource=%p, DstSubresource=%d, pDstBox=%p, pSrcData=%p, SrcRowPitch=%d, SrcDepthPitch=%d) stub\n",
          This, pDstResource, DstSubresource, pDstBox, pSrcData, SrcRowPitch, SrcDepthPitch); /* FIXME */
}

static void WINAPI ID3D10DeviceImpl_ClearRenderTargetView(ID3D10Device *iface,
                   ID3D10RenderTargetView *pRenderTargetView, const FLOAT ColorRGBA[4]) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pRenderTargetView=%p, ColorRGBA='%f,%f,%f,%f') stub\n",
          This, pRenderTargetView, ColorRGBA[0], ColorRGBA[1], ColorRGBA[2], ColorRGBA[3]);
}

static void WINAPI ID3D10DeviceImpl_ClearDepthStencilView(ID3D10Device *iface,
                   ID3D10DepthStencilView *pDepthStencilView, UINT ClearFlags, FLOAT Depth,
                   UINT8 Stencil) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pDepthStencilView=%p, ClearFlags=%d, Depth=%f) stub\n", This, pDepthStencilView,
          ClearFlags, Depth); /* FIXME */
}

static void WINAPI ID3D10DeviceImpl_GenerateMips(ID3D10Device *iface,
                   ID3D10ShaderResourceView *pShaderResourceView) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pShaderResourceView=%p) stub\n", This, pShaderResourceView);
}

static void WINAPI ID3D10DeviceImpl_ResolveSubresource(ID3D10Device *iface,
                   ID3D10Resource *pDstResource, UINT DstSubresource, ID3D10Resource *pSrcResource,
                   UINT SrcSubresource, DXGI_FORMAT Format) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pDstResource=%p, DstSubresource=%d, pSrcResource=%p, SrcSubresource=%d, Format=%d (%s)) stub\n",
          This, pDstResource, DstSubresource, pSrcResource, SrcSubresource, (UINT) Format, debug_dxgi_format(Format));
}

static void WINAPI ID3D10DeviceImpl_VSGetConstantBuffers(ID3D10Device *iface, UINT StartSlot,
                   UINT NumBuffers, ID3D10Buffer **ppConstantBuffers) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumBuffers=%d, ppConstantBuffers=%p) stub\n",
          This, StartSlot, NumBuffers, ppConstantBuffers);
}

static void WINAPI ID3D10DeviceImpl_PSGetShaderResources(ID3D10Device *iface, UINT StartSlot,
                   UINT NumViews, ID3D10ShaderResourceView **ppShaderResourceViews) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumViews=%d, ppShaderResourceViews=%p) stub\n",
          This, StartSlot, NumViews, ppShaderResourceViews);
}

static void WINAPI ID3D10DeviceImpl_PSGetShader(ID3D10Device *iface, ID3D10PixelShader **ppPixelShader) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, ppPixelShader=%p) stub\n", This, ppPixelShader);
}

static void WINAPI ID3D10DeviceImpl_PSGetSamplers(ID3D10Device *iface, UINT StartSlot, UINT NumSamplers,
                   ID3D10SamplerState **ppSamplers) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumSamplers=%d, ppSamplers=%p) stub\n", This, StartSlot, NumSamplers,
          ppSamplers);
}

static void WINAPI ID3D10DeviceImpl_VSGetShader(ID3D10Device *iface, ID3D10VertexShader **ppVertexShader) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, ppVertexShader=%p) stub\n", This, ppVertexShader);
}

static void WINAPI ID3D10DeviceImpl_PSGetConstantBuffers(ID3D10Device *iface, UINT StartSlot,
                   UINT NumBuffers, ID3D10Buffer **ppConstantBuffers) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumBuffers=%d, ppConstantBuffers=%p) stub\n",
          This, StartSlot, NumBuffers, ppConstantBuffers);
}

static void WINAPI ID3D10DeviceImpl_IAGetInputLayout(ID3D10Device *iface,
                   ID3D10InputLayout **ppInputLayout) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, ppInputLayout=%p) stub\n", This, ppInputLayout);
}

static void WINAPI ID3D10DeviceImpl_IAGetVertexBuffers(ID3D10Device *iface, UINT StartSlot, UINT NumBuffers,
                   ID3D10Buffer **ppVertexBuffers, UINT *pStrides, UINT *pOffsets) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumBuffers=%d, ppVertexBuffers=%p, pStrides=%p, pOffsets=%p) stub\n",
          This, StartSlot, NumBuffers, ppVertexBuffers, pStrides, pOffsets);
}

static void WINAPI ID3D10DeviceImpl_IAGetIndexBuffer(ID3D10Device *iface, ID3D10Buffer **pIndexBuffer,
                   DXGI_FORMAT *Format, UINT *Offset) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pIndexBuffer=%p, Format=%p, Offset=%p) stub\n", This, pIndexBuffer, Format, Offset);
          /* FIXME */
}

static void WINAPI ID3D10DeviceImpl_GSGetConstantBuffers(ID3D10Device *iface, UINT StartSlot,
                   UINT NumBuffers, ID3D10Buffer **ppConstantBuffers) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumBuffers=%d, ppConstantBuffers=%p) stub\n",
          This, StartSlot, NumBuffers, ppConstantBuffers);
}

static void WINAPI ID3D10DeviceImpl_GSGetShader(ID3D10Device *iface,
                   ID3D10GeometryShader **ppGeometryShader) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, ppGeometryShader=%p) stub\n", This, ppGeometryShader);
}

static void WINAPI ID3D10DeviceImpl_IAGetPrimitiveTopology(ID3D10Device *iface,
                   D3D10_PRIMITIVE_TOPOLOGY *pTopology) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pTopology=%p) stub\n", This, pTopology);
}

static void WINAPI ID3D10DeviceImpl_VSGetShaderResources(ID3D10Device *iface,
                   UINT StartSlot, UINT NumViews, ID3D10ShaderResourceView **ppShaderResourceViews) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumViews=%d, ppShaderResourceViews=%p) stub\n",
          This, StartSlot, NumViews, ppShaderResourceViews);
}

static void WINAPI ID3D10DeviceImpl_VSGetSamplers(ID3D10Device *iface, UINT StartSlot, UINT NumSamplers,
                   ID3D10SamplerState **ppSamplers) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumSamplers=%d, ppSamplers=%p) stub\n", This, StartSlot, NumSamplers,
          ppSamplers);
}

static void WINAPI ID3D10DeviceImpl_GetPredication(ID3D10Device *iface, ID3D10Predicate **ppPredicate,
                   BOOL *pPredicateValue) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, ppPredicate=%p, pPredicateValue=%p) stub\n", This, ppPredicate, pPredicateValue);
}

static void WINAPI ID3D10DeviceImpl_GSGetShaderResources(ID3D10Device *iface, UINT StartSlot, UINT NumViews,
                   ID3D10ShaderResourceView **ppShaderResourceViews) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumViews=%d, ppShaderResourceViews=%p) stub\n",
          This, StartSlot, NumViews, ppShaderResourceViews);
}

static void WINAPI ID3D10DeviceImpl_GSGetSamplers(ID3D10Device *iface, UINT StartSlot, UINT NumSamplers,
                   ID3D10SamplerState **ppSamplers) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, StartSlot=%d, NumSamplers=%d, ppSamplers=%p) stub\n", This, StartSlot, NumSamplers,
          ppSamplers);
}

static void WINAPI ID3D10DeviceImpl_OMGetRenderTargets(ID3D10Device *iface, UINT NumViews,
                   ID3D10RenderTargetView **ppRenderTargetViews,
                   ID3D10DepthStencilView **ppDepthStencilView) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, NumViews=%d, ppRenderTargetViews=%p, ppDepthStencilView=%p) stub\n",
          This, NumViews, ppRenderTargetViews, ppDepthStencilView);
}

static void WINAPI ID3D10DeviceImpl_OMGetBlendState(ID3D10Device *iface, ID3D10BlendState **ppBlendState,
                   FLOAT BlendFactor[4], UINT *pSampleMask) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, ppBlendState=%p, BlendFactor='%f,%f,%f,%f', pSampleMask=%p) stub\n",
          This, ppBlendState, BlendFactor[0], BlendFactor[1], BlendFactor[2], BlendFactor[3], pSampleMask);
          /* FIXME */
}

static void WINAPI ID3D10DeviceImpl_OMGetDepthStencilState(ID3D10Device *iface,
                   ID3D10DepthStencilState **ppDepthStencilState, UINT *pStencilRef) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, ppDepthStencilState=%p, pStencilRef=%p) stub\n", This, ppDepthStencilState,
          pStencilRef);
}

static void WINAPI ID3D10DeviceImpl_SOGetTargets(ID3D10Device *iface, UINT NumBuffers,
                   ID3D10Buffer **ppSOTargets, UINT *pOffsets) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, NumBuffers=%d, ppSOTargets=%p, pOffsets=%p) stub\n", This, NumBuffers, ppSOTargets,
          pOffsets);
}

static void WINAPI ID3D10DeviceImpl_RSGetState(ID3D10Device *iface,
                   ID3D10RasterizerState **ppRasterizerState) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, ppRasterizerState=%p) stub\n", This, ppRasterizerState);
}

static void WINAPI ID3D10DeviceImpl_RSGetViewports(ID3D10Device *iface, UINT *NumViewports,
                   D3D10_VIEWPORT *pViewports) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, NumViewports=%p, pViewports=%p) stub\n", This, NumViewports, pViewports);
}

static void WINAPI ID3D10DeviceImpl_RSGetScissorRects(ID3D10Device *iface, UINT *NumRects,
                   D3D10_RECT *pRects) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, NumRects=%p, pRects=%p) stub\n", This, NumRects, pRects); /* FIXME */
}

static HRESULT WINAPI ID3D10DeviceImpl_GetDeviceRemovedReason(ID3D10Device *iface) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p) stub\n", This);

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_SetExceptionMode(ID3D10Device *iface, UINT RaiseFlags) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, RaiseFlags=%d) stub\n", This, RaiseFlags); /* FIXME */

    return S_OK;
}

static UINT WINAPI ID3D10DeviceImpl_GetExceptionMode(ID3D10Device *iface) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p) stub\n", This);

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_GetPrivateData(ID3D10Device *iface, REFGUID guid,
                      UINT *pDataSize, void *pData) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("This=%p, guid=%s, pDataSize=%p, pData=%p) stub\n", This, debugstr_guid(guid), pDataSize, pData);

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_SetPrivateData(ID3D10Device *iface, REFGUID guid, UINT DataSize,
                      const void *pData) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("This=%p, guid=%s, DataSize=%d, pData=%p) stub\n", This, debugstr_guid(guid), DataSize, pData);

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_SetPrivateDataInterface(ID3D10Device *iface, REFGUID guid,
                      const IUnknown *pData) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("This=%p, guid=%s, pData=%p) stub\n", This, debugstr_guid(guid), pData);

    return S_OK;
}

static void WINAPI ID3D10DeviceImpl_ClearState(ID3D10Device *iface) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p) stub\n", This);
}

static void WINAPI ID3D10DeviceImpl_Flush(ID3D10Device *iface) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p) stub\n", This);
}

static HRESULT WINAPI ID3D10DeviceImpl_CreateBuffer(ID3D10Device *iface, const D3D10_BUFFER_DESC *pDesc,
                      const D3D10_SUBRESOURCE_DATA *pInitialData, ID3D10Buffer **ppBuffer) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ID3D10BufferImpl *object;

    FIXME("(This=%p, pDesc=%p, pInitialData=%p, ppBuffer=%p) stub\n", This, pDesc, pInitialData, ppBuffer);

    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
    if(!object) {
        ERR("Out of memory when allocating a D3D10Buffer implementation structure\n");
        ppBuffer = NULL;
        return E_OUTOFMEMORY;
     }
    object->lpVtbl = &D3D10Buffer_Vtbl;
    object->resource.child.ref = 1;
    *ppBuffer = (ID3D10Buffer *) object;
    TRACE("Created Buffer at %p\n", object);

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CreateTexture1D(ID3D10Device *iface,
                      const D3D10_TEXTURE1D_DESC *pDesc, const D3D10_SUBRESOURCE_DATA *pInitialData,
                      ID3D10Texture1D **ppTexture1D) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ID3D10Texture1DImpl *object;

    FIXME("(This=%p, pDesc=%p, pInitialData=%p, ppTexture1D=%p) stub\n", This,
           pDesc, pInitialData, ppTexture1D);

    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
    if(!object) {
        ERR("Out of memory when allocating a D3D10Texture1D implementation structure\n");
        ppTexture1D = NULL;
        return E_OUTOFMEMORY;
     }
    object->lpVtbl = &D3D10Texture1D_Vtbl;
    object->resource.child.ref = 1;
    *ppTexture1D = (ID3D10Texture1D *) object;
    TRACE("Created Texture1D at %p\n", object);

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CreateTexture2D(ID3D10Device *iface,
                      const D3D10_TEXTURE2D_DESC *pDesc, const D3D10_SUBRESOURCE_DATA *pInitialData,
                      ID3D10Texture2D **ppTexture2D) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ID3D10Texture2DImpl *object;

    FIXME("(This=%p, pDesc=%p, pInitialData=%p, ppTexture2D=%p) stub\n", This, pDesc, pInitialData,
           ppTexture2D);

    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
    if(!object) {
        ERR("Out of memory when allocating a D3D10Texture2D implementation structure\n");
        ppTexture2D = NULL;
        return E_OUTOFMEMORY;
     }
    object->lpVtbl = &D3D10Texture2D_Vtbl;
    object->resource.child.ref = 1;
    *ppTexture2D = (ID3D10Texture2D *) object;
    TRACE("Created Texture2D at %p\n", object);

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CreateTexture3D(ID3D10Device *iface,
                      const D3D10_TEXTURE3D_DESC *pDesc, const D3D10_SUBRESOURCE_DATA *pInitialData,
                      ID3D10Texture3D **ppTexture3D) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ID3D10Texture3DImpl *object;

    FIXME("(This=%p, pDesc=%p, pInitialData=%p, ppTexture3D=%p) stub\n", This, pDesc, pInitialData,
           ppTexture3D);

    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
    if(!object) {
        ERR("Out of memory when allocating a D3D10Texture3D implementation structure\n");
        ppTexture3D = NULL;
        return E_OUTOFMEMORY;
     }
    object->lpVtbl = &D3D10Texture3D_Vtbl;
    object->resource.child.ref = 1;
    *ppTexture3D = (ID3D10Texture3D *) object;
    TRACE("Created Texture3D at %p\n", object);

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CreateShaderResourceView(ID3D10Device *iface,
                      ID3D10Resource *pResource, const D3D10_SHADER_RESOURCE_VIEW_DESC *pDesc,
                      ID3D10ShaderResourceView **ppSRView) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ID3D10ShaderResourceViewImpl *object;

    FIXME("(This=%p, pResource=%p, pDesc=%p, ppSRView=%p) stub\n", This, pResource, pDesc, ppSRView);
    
    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
    if(!object) {
        ERR("Out of memory when allocating a D3D10ShaderResourceView implementation structure\n");
        ppSRView = NULL;
        return E_OUTOFMEMORY;
     }
    object->lpVtbl = &D3D10ShaderResourceView_Vtbl;
    object->child.ref = 1;
    *ppSRView = (ID3D10ShaderResourceView *) object;
    TRACE("Created ShaderResourceView at %p\n", object);

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CreateRenderTargetView(ID3D10Device *iface,
                      ID3D10Resource *pResource, const D3D10_RENDER_TARGET_VIEW_DESC *pDesc,
                      ID3D10RenderTargetView **ppRTView) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ID3D10RenderTargetViewImpl *object;

    FIXME("(This=%p, pResource=%p, pDesc=%p, ppRTView=%p) stub\n", This, pResource, pDesc, ppRTView);
    
    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
    if(!object) {
        ERR("Out of memory when allocating a D3D10RenderTargetView implementation structure\n");
        ppRTView = NULL;
        return E_OUTOFMEMORY;
     }
    object->lpVtbl = &D3D10RenderTargetView_Vtbl;
    object->view.child.ref = 1;
    *ppRTView = (ID3D10RenderTargetView *) object;
    TRACE("Created RenderTargetView at %p\n", object);    

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CreateDepthStencilView(ID3D10Device *iface,
                      ID3D10Resource *pResource, const D3D10_DEPTH_STENCIL_VIEW_DESC *pDesc,
                      ID3D10DepthStencilView **ppDepthStencilView) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ID3D10DepthStencilViewImpl *object;

    FIXME("(This=%p, pResource=%p, pDesc=%p, ppDepthStencilView=%p) stub\n", This, pResource,
            pDesc, ppDepthStencilView);
    
    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
    if(!object) {
        ERR("Out of memory when allocating a D3D10DepthStencilView implementation structure\n");
        ppDepthStencilView = NULL;
        return E_OUTOFMEMORY;
     }
    object->lpVtbl = &D3D10DepthStencilView_Vtbl;
    object->view.child.ref = 1;
    *ppDepthStencilView = (ID3D10DepthStencilView *) object;
    TRACE("Created DepthStencilView at %p\n", object);    

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CreateInputLayout(ID3D10Device *iface,
                      const D3D10_INPUT_ELEMENT_DESC *pInputElementDescs, UINT NumElements,
                      const void *pShaderBytecodeWithInputSignature, SIZE_T BytecodeLength,
                      ID3D10InputLayout **ppInputLayout) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ID3D10InputLayoutImpl *object;

    FIXME("(This=%p, pInputElementDescs=%p, NumElements=%d, pShaderBytecodeWithInputSignature=%p, BytecodeLength=%d, ppInputLayout=%p) stub\n",
          This, pInputElementDescs, NumElements, pShaderBytecodeWithInputSignature, (int) BytecodeLength,
          ppInputLayout);
    
    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
    if(!object) {
        ERR("Out of memory when allocating a D3D10InputLayout implementation structure\n");
        ppInputLayout = NULL;
        return E_OUTOFMEMORY;
     }
    object->lpVtbl = &D3D10InputLayout_Vtbl;
    object->child.ref = 1;
    *ppInputLayout = (ID3D10InputLayout *) object;
    TRACE("Created InputLayout at %p\n", object);    

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CreateVertexShader(ID3D10Device *iface, const void *pShaderBytecode,
                      SIZE_T BytecodeLength, ID3D10VertexShader **ppVertexShader) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ID3D10VertexShaderImpl *object;

    FIXME("(This=%p, pShaderBytecode=%p, BytecodeLength=%d, ppVertexShader=%p) stub\n",
          This, pShaderBytecode, (int) BytecodeLength, ppVertexShader);
    
    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
    if(!object) {
        ERR("Out of memory when allocating a D3D10VertexShader implementation structure\n");
        ppVertexShader = NULL;
        return E_OUTOFMEMORY;
     }
    object->lpVtbl = &D3D10VertexShader_Vtbl;
    object->child.ref = 1;
    *ppVertexShader = (ID3D10VertexShader *) object;
    TRACE("Created VertexShader at %p\n", object);    

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CreateGeometryShader(ID3D10Device *iface,
                      const void *pShaderBytecode, SIZE_T BytecodeLength,
                      ID3D10GeometryShader **ppGeometryShader) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ID3D10GeometryShaderImpl *object;

    FIXME("(This=%p, pShaderBytecode=%p, BytecodeLength=%d, ppGeometryShader=%p) stub\n",
           This, pShaderBytecode, (int) BytecodeLength, ppGeometryShader);
    
    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
    if(!object) {
        ERR("Out of memory when allocating a D3D10GeometryShader implementation structure\n");
        ppGeometryShader = NULL;
        return E_OUTOFMEMORY;
     }
    object->lpVtbl = &D3D10GeometryShader_Vtbl;
    object->child.ref = 1;
    *ppGeometryShader = (ID3D10GeometryShader *) object;
    TRACE("Created GeometryShader at %p\n", object);    

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CreateGeometryShaderWithStreamOutput(ID3D10Device *iface,
                      const void *pShaderBytecode, SIZE_T BytecodeLength,
                      const D3D10_SO_DECLARATION_ENTRY *pSODeclaration, UINT NumEntries,
                      UINT OutputStreamStride, ID3D10GeometryShader **ppGeometryShader) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ID3D10GeometryShaderImpl *object;

    FIXME("(This=%p, pShaderBytecode=%p, BytecodeLength=%d, pSODeclaration=%p, NumEntries=%d, OutputStreamStride=%d, ppGeometryShader=%p) stub\n",
          This, pShaderBytecode, (int) BytecodeLength, pSODeclaration, NumEntries, OutputStreamStride,
          ppGeometryShader);
    
    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
    if(!object) {
        ERR("Out of memory when allocating a D3D10GeometryShader implementation structure\n");
        ppGeometryShader = NULL;
        return E_OUTOFMEMORY;
     }
    object->lpVtbl = &D3D10GeometryShader_Vtbl;
    object->child.ref = 1;
    *ppGeometryShader = (ID3D10GeometryShader *) object;
    TRACE("Created GeometryShader at %p\n", object);    

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CreatePixelShader(ID3D10Device *iface,
                      const void *pShaderBytecode, SIZE_T BytecodeLength,
                      ID3D10PixelShader **ppPixelShader) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ID3D10PixelShaderImpl *object;

    FIXME("(This=%p, pShaderBytecode=%p, BytecodeLength=%d, ppPixelShader=%p) stub\n",
          This, pShaderBytecode, (int) BytecodeLength, ppPixelShader);
    
    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
    if(!object) {
        ERR("Out of memory when allocating a D3D10PixelShader implementation structure\n");
        ppPixelShader = NULL;
        return E_OUTOFMEMORY;
     }
    object->lpVtbl = &D3D10PixelShader_Vtbl;
    object->child.ref = 1;
    *ppPixelShader = (ID3D10PixelShader *) object;
    TRACE("Created PixelShader at %p\n", object);    

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CreateBlendState(ID3D10Device *iface,
                      const D3D10_BLEND_DESC *pBlendStateDesc, ID3D10BlendState **ppBlendState) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ID3D10BlendStateImpl *object;

    FIXME("(This=%p, pBlendStateDesc=%p, ppBlendState=%p) stub\n", This, pBlendStateDesc,
           ppBlendState); /* FIXME */

    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
    if(!object) {
        ERR("Out of memory when allocating a D3D10BlendState implementation structure\n");
        ppBlendState = NULL;
        return E_OUTOFMEMORY;
     }
    object->lpVtbl = &D3D10BlendState_Vtbl;
    object->child.ref = 1;
    *ppBlendState = (ID3D10BlendState *) object;
    TRACE("Created BlendState at %p\n", object);

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CreateDepthStencilState(ID3D10Device *iface,
                      const D3D10_DEPTH_STENCIL_DESC *pDepthStencilDesc,
                      ID3D10DepthStencilState **ppDepthStencilState) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ID3D10DepthStencilStateImpl *object;

    FIXME("(This=%p, pDepthStencilDesc=%p, ppDepthStencilState=%p) stub\n", This, pDepthStencilDesc,
           ppDepthStencilState);

    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
    if(!object) {
        ERR("Out of memory when allocating a D3D10DepthStencilState implementation structure\n");
        ppDepthStencilState = NULL;
        return E_OUTOFMEMORY;
     }
    object->lpVtbl = &D3D10DepthStencilState_Vtbl;
    object->child.ref = 1;
    *ppDepthStencilState = (ID3D10DepthStencilState *) object;
    TRACE("Created DepthStencilState at %p\n", object);

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CreateRasterizerState(ID3D10Device *iface,
                      const D3D10_RASTERIZER_DESC *pRasterizerDesc,
                      ID3D10RasterizerState **ppRasterizerState) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ID3D10RasterizerStateImpl *object;

    FIXME("(This=%p, pRasterizerDesc=%p, ppRasterizerState=%p) stub\n", This, pRasterizerDesc,
            ppRasterizerState);

    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
    if(!object) {
        ERR("Out of memory when allocating a D3D10RasterizerState implementation structure\n");
        ppRasterizerState = NULL;
        return E_OUTOFMEMORY;
     }
    object->lpVtbl = &D3D10RasterizerState_Vtbl;
    object->child.ref = 1;
    *ppRasterizerState = (ID3D10RasterizerState *) object;
    TRACE("Created RasterizerState at %p\n", object);

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CreateSamplerState(ID3D10Device *iface,
                      const D3D10_SAMPLER_DESC *pSamplerDesc,
                      ID3D10SamplerState **ppSamplerState) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ID3D10SamplerStateImpl *object;

    FIXME("(This=%p, pSamplerDesc=%p, ppSamplerState=%p) stub\n", This, pSamplerDesc, ppSamplerState);

    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
    if(!object) {
        ERR("Out of memory when allocating a D3D10SamplerState implementation structure\n");
        ppSamplerState = NULL;
        return E_OUTOFMEMORY;
     }
    object->lpVtbl = &D3D10SamplerState_Vtbl;
    object->child.ref = 1;
    *ppSamplerState = (ID3D10SamplerState *) object;
    TRACE("Created SamplerState at %p\n", object);

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CreateQuery(ID3D10Device *iface, const D3D10_QUERY_DESC *pQueryDesc,
                      ID3D10Query **ppQuery) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ID3D10QueryImpl *object;

    FIXME("(This=%p, pQueryDesc=%p, ppQuery=%p) stub\n", This, pQueryDesc, ppQuery);

    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
    if(!object) {
        ERR("Out of memory when allocating a D3D10Query implementation structure\n");
        ppQuery = NULL;
        return E_OUTOFMEMORY;
     }
    object->lpVtbl = &D3D10Query_Vtbl;
    object->query.asynchronous.child.ref = 1;
    *ppQuery = (ID3D10Query *) object;
    TRACE("Created Query at %p\n", object);

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CreatePredicate(ID3D10Device *iface,
                      const D3D10_QUERY_DESC *pPredicateDesc, ID3D10Predicate **ppPredicate) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ID3D10PredicateImpl *object;

    FIXME("(This=%p, pPredicateDesc=%p, ppPredicate=%p) stub\n", This, pPredicateDesc, ppPredicate);

    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
    if(!object) {
        ERR("Out of memory when allocating a D3D10Predicate implementation structure\n");
        ppPredicate = NULL;
        return E_OUTOFMEMORY;
     }
    object->lpVtbl = &D3D10Predicate_Vtbl;
    object->query.asynchronous.child.ref = 1;
    *ppPredicate = (ID3D10Predicate *) object;
    TRACE("Created Predicate at %p\n", object);

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CreateCounter(ID3D10Device *iface,
                      const D3D10_COUNTER_DESC *pCounterDesc, ID3D10Counter **ppCounter) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;
    ID3D10CounterImpl *object;

    FIXME("(This=%p, pCounterDesc=%p, ppCounter=%p) stub\n", This, pCounterDesc, ppCounter);

    object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object));
    if(!object) {
        ERR("Out of memory when allocating a D3D10Counter implementation structure\n");
        ppCounter = NULL;
        return E_OUTOFMEMORY;
     }
    object->lpVtbl = &D3D10Counter_Vtbl;
    object->asynchronous.child.ref = 1;
    *ppCounter = (ID3D10Counter *) object;
    TRACE("Created Counter at %p\n", object);

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CheckFormatSupport(ID3D10Device *iface,
                      DXGI_FORMAT Format, UINT *pFormatSupport) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, Format=%d, pFormatSupport=%p) stub\n", This, (int) Format, pFormatSupport);

    return S_OK;
}

static HRESULT WINAPI ID3D10DeviceImpl_CheckMultisampleQualityLevels(ID3D10Device *iface,
                      DXGI_FORMAT Format, UINT SampleCount, UINT *pNumQualityLevels) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, Format=%d, SampleCount=%d, pNumQualityLevels=%p) stub\n", This,
          (int) Format, SampleCount, pNumQualityLevels);

    return S_OK;
}

static void WINAPI ID3D10DeviceImpl_CheckCounterInfo(ID3D10Device *iface,
                   D3D10_COUNTER_INFO *pCounterInfo) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pCounterInfo=%p) stub\n", This, pCounterInfo);
}

static HRESULT WINAPI ID3D10DeviceImpl_CheckCounter(ID3D10Device *iface, const D3D10_COUNTER_DESC *pDesc,
                      D3D10_COUNTER_TYPE *pType, UINT *pActiveCounters, LPSTR szName, UINT *pNameLength,
                      LPSTR szUnits, UINT *pUnitsLength, LPSTR szDescription, UINT *pDescriptionLength) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, pDesc=%p, pType=%p, pActiveCounters=%p, szName=%s, pNameLength=%p, szUnits=%s, pUnitsLength=%p, szDescription=%s, pDescriptionLength=%p) stub\n",
          This, pDesc, pType, pActiveCounters, szName, pNameLength, szUnits, pUnitsLength, szDescription,
          pDescriptionLength);

    return S_OK;
}

static UINT WINAPI ID3D10DeviceImpl_GetCreationFlags(ID3D10Device *iface) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p) stub\n", This);

    return 0;
}

static HRESULT WINAPI ID3D10DeviceImpl_OpenSharedResource(ID3D10Device *iface, HANDLE hResource,
                      REFIID ReturnedInterface, void **ppResource) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, hResource=%p, ReturnedInterface=%s, ppResource=%p) stub\n", This,
            hResource, debugstr_guid(ReturnedInterface), ppResource);

    return S_OK;
}

static void WINAPI ID3D10DeviceImpl_SetTextFilterSize(ID3D10Device *iface, UINT Width, UINT Height) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, Width=%d, Height=%d) stub\n", This, Width, Height);
}

static void WINAPI ID3D10DeviceImpl_GetTextFilterSize(ID3D10Device *iface, UINT *pWidth, UINT *pHeight) {
    ID3D10DeviceImpl *This = (ID3D10DeviceImpl *)iface;

    FIXME("(This=%p, Width=%p, Height=%p) stub\n", This, pWidth, pHeight);
}

const ID3D10DeviceVtbl D3D10Device_Vtbl =
{
    ID3D10DeviceImpl_QueryInterface,
    ID3D10DeviceImpl_AddRef,
    ID3D10DeviceImpl_Release,
    ID3D10DeviceImpl_VSSetConstantBuffers,
    ID3D10DeviceImpl_PSSetShaderResources,
    ID3D10DeviceImpl_PSSetShader,
    ID3D10DeviceImpl_PSSetSamplers,
    ID3D10DeviceImpl_VSSetShader,
    ID3D10DeviceImpl_DrawIndexed,
    ID3D10DeviceImpl_Draw,
    ID3D10DeviceImpl_PSSetConstantBuffers,
    ID3D10DeviceImpl_IASetInputLayout,
    ID3D10DeviceImpl_IASetVertexBuffers,
    ID3D10DeviceImpl_IASetIndexBuffer,
    ID3D10DeviceImpl_DrawIndexedInstanced,
    ID3D10DeviceImpl_DrawInstanced,
    ID3D10DeviceImpl_GSSetConstantBuffers,
    ID3D10DeviceImpl_GSSetShader,
    ID3D10DeviceImpl_IASetPrimitiveTopology,
    ID3D10DeviceImpl_VSSetShaderResources,
    ID3D10DeviceImpl_VSSetSamplers,
    ID3D10DeviceImpl_SetPredication,
    ID3D10DeviceImpl_GSSetShaderResources,
    ID3D10DeviceImpl_GSSetSamplers,
    ID3D10DeviceImpl_OMSetRenderTargets,
    ID3D10DeviceImpl_OMSetBlendState,
    ID3D10DeviceImpl_OMSetDepthStencilState,
    ID3D10DeviceImpl_SOSetTargets,
    ID3D10DeviceImpl_DrawAuto,
    ID3D10DeviceImpl_RSSetState,
    ID3D10DeviceImpl_RSSetViewports,
    ID3D10DeviceImpl_RSSetScissorRects,
    ID3D10DeviceImpl_CopySubresourceRegion,
    ID3D10DeviceImpl_CopyResource,
    ID3D10DeviceImpl_UpdateSubresource,
    ID3D10DeviceImpl_ClearRenderTargetView,
    ID3D10DeviceImpl_ClearDepthStencilView,
    ID3D10DeviceImpl_GenerateMips,
    ID3D10DeviceImpl_ResolveSubresource,
    ID3D10DeviceImpl_VSGetConstantBuffers,
    ID3D10DeviceImpl_PSGetShaderResources,
    ID3D10DeviceImpl_PSGetShader,
    ID3D10DeviceImpl_PSGetSamplers,
    ID3D10DeviceImpl_VSGetShader,
    ID3D10DeviceImpl_PSGetConstantBuffers,
    ID3D10DeviceImpl_IAGetInputLayout,
    ID3D10DeviceImpl_IAGetVertexBuffers,
    ID3D10DeviceImpl_IAGetIndexBuffer,
    ID3D10DeviceImpl_GSGetConstantBuffers,
    ID3D10DeviceImpl_GSGetShader,
    ID3D10DeviceImpl_IAGetPrimitiveTopology,
    ID3D10DeviceImpl_VSGetShaderResources,
    ID3D10DeviceImpl_VSGetSamplers,
    ID3D10DeviceImpl_GetPredication,
    ID3D10DeviceImpl_GSGetShaderResources,
    ID3D10DeviceImpl_GSGetSamplers,
    ID3D10DeviceImpl_OMGetRenderTargets,
    ID3D10DeviceImpl_OMGetBlendState,
    ID3D10DeviceImpl_OMGetDepthStencilState,
    ID3D10DeviceImpl_SOGetTargets,
    ID3D10DeviceImpl_RSGetState,
    ID3D10DeviceImpl_RSGetViewports,
    ID3D10DeviceImpl_RSGetScissorRects,
    ID3D10DeviceImpl_GetDeviceRemovedReason,
    ID3D10DeviceImpl_SetExceptionMode,
    ID3D10DeviceImpl_GetExceptionMode,
    ID3D10DeviceImpl_GetPrivateData,
    ID3D10DeviceImpl_SetPrivateData,
    ID3D10DeviceImpl_SetPrivateDataInterface,
    ID3D10DeviceImpl_ClearState,
    ID3D10DeviceImpl_Flush,
    ID3D10DeviceImpl_CreateBuffer,
    ID3D10DeviceImpl_CreateTexture1D,
    ID3D10DeviceImpl_CreateTexture2D,
    ID3D10DeviceImpl_CreateTexture3D,
    ID3D10DeviceImpl_CreateShaderResourceView,
    ID3D10DeviceImpl_CreateRenderTargetView,
    ID3D10DeviceImpl_CreateDepthStencilView,
    ID3D10DeviceImpl_CreateInputLayout,
    ID3D10DeviceImpl_CreateVertexShader,
    ID3D10DeviceImpl_CreateGeometryShader,
    ID3D10DeviceImpl_CreateGeometryShaderWithStreamOutput,
    ID3D10DeviceImpl_CreatePixelShader,
    ID3D10DeviceImpl_CreateBlendState,
    ID3D10DeviceImpl_CreateDepthStencilState,
    ID3D10DeviceImpl_CreateRasterizerState,
    ID3D10DeviceImpl_CreateSamplerState,
    ID3D10DeviceImpl_CreateQuery,
    ID3D10DeviceImpl_CreatePredicate,
    ID3D10DeviceImpl_CreateCounter,
    ID3D10DeviceImpl_CheckFormatSupport,
    ID3D10DeviceImpl_CheckMultisampleQualityLevels,
    ID3D10DeviceImpl_CheckCounterInfo,
    ID3D10DeviceImpl_CheckCounter,
    ID3D10DeviceImpl_GetCreationFlags,
    ID3D10DeviceImpl_OpenSharedResource,
    ID3D10DeviceImpl_SetTextFilterSize,
    ID3D10DeviceImpl_GetTextFilterSize
};
