/*
 * Direct3D 10
 *
 * Copyright 2007 Andras Kovacs
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#include "config.h"

#include "d3d10_private.h"

WINE_DEFAULT_DEBUG_CHANNEL(d3d10);

/* ID3D10InputLayout Implementation */
static HRESULT WINAPI ID3D10InputLayoutImpl_QueryInterface(ID3D10InputLayout *iface, REFIID riid,
               void **obj) {
    ID3D10InputLayoutImpl *This = (ID3D10InputLayoutImpl *)iface;

    TRACE("(%p)->(%s,%p)\n", This, debugstr_guid(riid), obj);
    if(IsEqualGUID(riid, &IID_IUnknown) ||
       IsEqualGUID(riid, &IID_ID3D10InputLayout)) {
        IUnknown_AddRef(iface);
        *obj = This;
        return S_OK;
    }

    WARN("(%p)->(%s,%p),not found\n", This, debugstr_guid(riid), obj);
    *obj = NULL;
    return E_NOINTERFACE;
}

static ULONG WINAPI ID3D10InputLayoutImpl_AddRef(ID3D10InputLayout *iface) {
    ID3D10InputLayoutImpl *This = (ID3D10InputLayoutImpl *)iface;
    ULONG ref = InterlockedIncrement(&This->child.ref);

    TRACE("(%p) : AddRef from %d\n", This, ref - 1);
    return ref;
}

static ULONG WINAPI ID3D10InputLayoutImpl_Release(ID3D10InputLayout *iface) {
    ID3D10InputLayoutImpl *This = (ID3D10InputLayoutImpl *)iface;
    ULONG ref;

    ref = InterlockedDecrement(&This->child.ref);
    TRACE("(%p) : ReleaseRef to %d\n", This, ref);

    if (ref == 0) {
        HeapFree(GetProcessHeap(), 0, This);
    }

    return ref;
}

static void WINAPI ID3D10InputLayoutImpl_GetDevice(ID3D10InputLayout *iface, ID3D10Device **ppDevice) {
    ID3D10DeviceChildImpl_GetDevice((ID3D10DeviceChild *) iface, ppDevice);
}

static HRESULT WINAPI ID3D10InputLayoutImpl_GetPrivateData(ID3D10InputLayout *iface, REFGUID guid,
                      UINT *pDataSize, void *pData) {
    return ID3D10DeviceChildImpl_GetPrivateData((ID3D10DeviceChild *) iface, guid, pDataSize, pData);
}

static HRESULT WINAPI ID3D10InputLayoutImpl_SetPrivateData(ID3D10InputLayout *iface, REFGUID guid,
                      UINT DataSize, const void *pData) {
    return ID3D10DeviceChildImpl_SetPrivateData((ID3D10DeviceChild *) iface, guid, DataSize, pData);
}

static HRESULT WINAPI ID3D10InputLayoutImpl_SetPrivateDataInterface(ID3D10InputLayout *iface, REFGUID guid,
                      const IUnknown *pData) {
    return ID3D10DeviceChildImpl_SetPrivateDataInterface((ID3D10DeviceChild *) iface, guid, pData);
}

const ID3D10InputLayoutVtbl D3D10InputLayout_Vtbl =
{
    ID3D10InputLayoutImpl_QueryInterface,
    ID3D10InputLayoutImpl_AddRef,
    ID3D10InputLayoutImpl_Release,
    ID3D10InputLayoutImpl_GetDevice,
    ID3D10InputLayoutImpl_GetPrivateData,
    ID3D10InputLayoutImpl_SetPrivateData,
    ID3D10InputLayoutImpl_SetPrivateDataInterface
};
