/*
 * Direct3D 10
 *
 * Copyright 2007 Andras Kovacs
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#include "config.h"

#include "d3d10_private.h"

WINE_DEFAULT_DEBUG_CHANNEL(d3d10);

static HRESULT WINAPI ID3D10MultithreadImpl_QueryInterface(ID3D10Multithread *iface, REFIID riid,
                      void **obj) {
    ID3D10MultithreadImpl *This = (ID3D10MultithreadImpl *)iface;

    TRACE("(%p)->(%s,%p)\n", This, debugstr_guid(riid), obj);
    if(IsEqualGUID(riid, &IID_IUnknown) ||
       IsEqualGUID(riid, &IID_ID3D10Multithread)) {
        IUnknown_AddRef(iface);
        *obj = This;
        return S_OK;
    }

    WARN("(%p)->(%s,%p),not found\n", This, debugstr_guid(riid), obj);
    *obj = NULL;
    return E_NOINTERFACE;
}

static ULONG WINAPI ID3D10MultithreadImpl_AddRef(ID3D10Multithread *iface) {
    ID3D10MultithreadImpl *This = (ID3D10MultithreadImpl *)iface;
    ULONG ref = InterlockedIncrement(&This->ref);

    TRACE("(%p) : AddRef from %d\n", This, ref - 1);
    return ref;
}

static ULONG WINAPI ID3D10MultithreadImpl_Release(ID3D10Multithread *iface) {
    ID3D10MultithreadImpl *This = (ID3D10MultithreadImpl *)iface;
    ULONG ref;

    ref = InterlockedDecrement(&This->ref);
    TRACE("(%p) : ReleaseRef to %d\n", This, ref);

    if (ref == 0) {
        HeapFree(GetProcessHeap(), 0, This);
    }

    return ref;
}

static void WINAPI ID3D10MultithreadImpl_Enter(ID3D10Multithread *iface) {
    ID3D10MultithreadImpl *This = (ID3D10MultithreadImpl *)iface;

    FIXME("(This=%p) stub\n", This);
}

static void WINAPI ID3D10MultithreadImpl_Leave(ID3D10Multithread *iface) {
    ID3D10MultithreadImpl *This = (ID3D10MultithreadImpl *)iface;

    FIXME("(This=%p) stub\n", This);
}

static BOOL WINAPI ID3D10MultithreadImpl_SetMultithreadProtected(ID3D10Multithread *iface,
            BOOL bMTProtect) {
    ID3D10MultithreadImpl *This = (ID3D10MultithreadImpl *)iface;

    FIXME("(This=%p, bMTProtect=%d) stub\n", This, bMTProtect);

    return TRUE;
}

static BOOL WINAPI ID3D10MultithreadImpl_GetMultithreadProtected(ID3D10Multithread *iface) {
    ID3D10MultithreadImpl *This = (ID3D10MultithreadImpl *)iface;

    FIXME("(This=%p) stub\n", This);

    return TRUE;
}


const ID3D10MultithreadVtbl D3D10Multithread_Vtbl =
{
    ID3D10MultithreadImpl_QueryInterface,
    ID3D10MultithreadImpl_AddRef,
    ID3D10MultithreadImpl_Release,
    ID3D10MultithreadImpl_Enter,
    ID3D10MultithreadImpl_Leave,
    ID3D10MultithreadImpl_SetMultithreadProtected,
    ID3D10MultithreadImpl_GetMultithreadProtected
};
