/*
 * Direct3D 10
 *
 * Copyright 2007 Andras Kovacs
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#include "config.h"

#include "d3d10_private.h"

WINE_DEFAULT_DEBUG_CHANNEL(d3d10);

/* ID3D10Query Implementation */
static HRESULT WINAPI ID3D10QueryImpl_QueryInterface(ID3D10Query *iface, REFIID riid, void **obj) {
    ID3D10QueryImpl *This = (ID3D10QueryImpl *)iface;

    TRACE("(%p)->(%s,%p)\n", This, debugstr_guid(riid), obj);
    if(IsEqualGUID(riid, &IID_IUnknown) ||
       IsEqualGUID(riid, &IID_ID3D10Query)) {
        IUnknown_AddRef(iface);
        *obj = This;
        return S_OK;
    }

    WARN("(%p)->(%s,%p),not found\n", This, debugstr_guid(riid), obj);
    *obj = NULL;
    return E_NOINTERFACE;
}

static ULONG WINAPI ID3D10QueryImpl_AddRef(ID3D10Query *iface) {
    ID3D10QueryImpl *This = (ID3D10QueryImpl *)iface;
    ULONG ref = InterlockedIncrement(&This->query.asynchronous.child.ref);

    TRACE("(%p) : AddRef from %d\n", This, ref - 1);
    return ref;
}

static ULONG WINAPI ID3D10QueryImpl_Release(ID3D10Query *iface) {
    ID3D10QueryImpl *This = (ID3D10QueryImpl *)iface;
    ULONG ref;

    ref = InterlockedDecrement(&This->query.asynchronous.child.ref);
    TRACE("(%p) : ReleaseRef to %d\n", This, ref);

    if (ref == 0) {
        HeapFree(GetProcessHeap(), 0, This);
    }

    return ref;
}

void WINAPI ID3D10QueryImpl_GetDesc(ID3D10Query *iface, D3D10_QUERY_DESC *pDesc) {
    ID3D10QueryImpl *This = (ID3D10QueryImpl *)iface;

    FIXME("(This=%p, pDesc=%p) stub\n", This, pDesc);
}

static void WINAPI ID3D10QueryImpl_Begin(ID3D10Query *iface) {
    ID3D10AsynchronousImpl_Begin((ID3D10Asynchronous *) iface);
}

static void WINAPI ID3D10QueryImpl_End(ID3D10Query *iface) {
    ID3D10AsynchronousImpl_End((ID3D10Asynchronous *) iface);
}

static HRESULT WINAPI ID3D10QueryImpl_GetData(ID3D10Query *iface, void *pData, UINT DataSize,
        UINT GetDataFlags) {
    return ID3D10AsynchronousImpl_GetData((ID3D10Asynchronous *) iface, pData, DataSize, GetDataFlags);
}

static UINT WINAPI ID3D10QueryImpl_GetDataSize(ID3D10Query *iface) {
    return ID3D10AsynchronousImpl_GetDataSize((ID3D10Asynchronous *) iface);
}

static void WINAPI ID3D10QueryImpl_GetDevice(ID3D10Query *iface, ID3D10Device **ppDevice) {
    ID3D10DeviceChildImpl_GetDevice((ID3D10DeviceChild *) iface, ppDevice);
}

static HRESULT WINAPI ID3D10QueryImpl_GetPrivateData(ID3D10Query *iface, REFGUID guid,
                      UINT *pDataSize, void *pData) {
    return ID3D10DeviceChildImpl_GetPrivateData((ID3D10DeviceChild *) iface, guid, pDataSize, pData);
}

static HRESULT WINAPI ID3D10QueryImpl_SetPrivateData(ID3D10Query *iface, REFGUID guid,
                      UINT DataSize, const void *pData) {
    return ID3D10DeviceChildImpl_SetPrivateData((ID3D10DeviceChild *) iface, guid, DataSize, pData);
}

static HRESULT WINAPI ID3D10QueryImpl_SetPrivateDataInterface(ID3D10Query *iface, REFGUID guid,
                      const IUnknown *pData) {
    return ID3D10DeviceChildImpl_SetPrivateDataInterface((ID3D10DeviceChild *) iface, guid, pData);
}

const ID3D10QueryVtbl D3D10Query_Vtbl =
{
    ID3D10QueryImpl_QueryInterface,
    ID3D10QueryImpl_AddRef,
    ID3D10QueryImpl_Release,
    ID3D10QueryImpl_GetDevice,
    ID3D10QueryImpl_GetPrivateData,
    ID3D10QueryImpl_SetPrivateData,
    ID3D10QueryImpl_SetPrivateDataInterface,
    ID3D10QueryImpl_Begin,
    ID3D10QueryImpl_End,
    ID3D10QueryImpl_GetData,
    ID3D10QueryImpl_GetDataSize,
    ID3D10QueryImpl_GetDesc
};
