/*
 * Direct3D 10
 *
 * Copyright 2007 Andras Kovacs
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#include "config.h"

#include "d3d10_private.h"

WINE_DEFAULT_DEBUG_CHANNEL(d3d10);

static HRESULT WINAPI ID3D10ResourceImpl_QueryInterface(ID3D10Resource *iface, REFIID riid, void **obj) {
    ID3D10ResourceImpl *This = (ID3D10ResourceImpl *)iface;

    TRACE("(%p)->(%s,%p)\n", This, debugstr_guid(riid), obj);
    if(IsEqualGUID(riid, &IID_IUnknown) ||
       IsEqualGUID(riid, &IID_ID3D10Resource)) {
        IUnknown_AddRef(iface);
        *obj = This;
        return S_OK;
    }

    WARN("(%p)->(%s,%p),not found\n", This, debugstr_guid(riid), obj);
    *obj = NULL;
    return E_NOINTERFACE;
}

static ULONG WINAPI ID3D10ResourceImpl_AddRef(ID3D10Resource *iface) {
    ID3D10ResourceImpl *This = (ID3D10ResourceImpl *)iface;
    ULONG ref = InterlockedIncrement(&This->resource.child.ref);

    TRACE("(%p) : AddRef from %d\n", This, ref - 1);
    return ref;
}

static ULONG WINAPI ID3D10ResourceImpl_Release(ID3D10Resource *iface) {
    ID3D10ResourceImpl *This = (ID3D10ResourceImpl *)iface;
    ULONG ref;

    ref = InterlockedDecrement(&This->resource.child.ref);
    TRACE("(%p) : ReleaseRef to %d\n", This, ref);

    if (ref == 0) {
        HeapFree(GetProcessHeap(), 0, This);
    }

    return ref;
}

void WINAPI ID3D10ResourceImpl_GetType(ID3D10Resource *iface, D3D10_RESOURCE_DIMENSION *rType) {
    FIXME("(rType=%p) stub\n", rType); /* FIXME */
}

void WINAPI ID3D10ResourceImpl_SetEvictionPriority(ID3D10Resource *iface, UINT EvictionPriority) {
    FIXME("(EvictionPriority=%d) stub\n", EvictionPriority);
}

UINT WINAPI ID3D10ResourceImpl_GetEvictionPriority(ID3D10Resource *iface) {
    FIXME("() stub\n");

    return 0; /* eg. DXGI_RESOURCE_PRIORITY_NORMAL */ /* FIXME */
}

static void WINAPI ID3D10ResourceImpl_GetDevice(ID3D10Resource *iface, ID3D10Device **ppDevice) {
    ID3D10DeviceChildImpl_GetDevice((ID3D10DeviceChild *) iface, ppDevice);
}

static HRESULT WINAPI ID3D10ResourceImpl_GetPrivateData(ID3D10Resource *iface, REFGUID guid,
                      UINT *pDataSize, void *pData) {
    return ID3D10DeviceChildImpl_GetPrivateData((ID3D10DeviceChild *) iface, guid, pDataSize, pData);
}

static HRESULT WINAPI ID3D10ResourceImpl_SetPrivateData(ID3D10Resource *iface, REFGUID guid,
                      UINT DataSize, const void *pData) {
    return ID3D10DeviceChildImpl_SetPrivateData((ID3D10DeviceChild *) iface, guid, DataSize, pData);
}

static HRESULT WINAPI ID3D10ResourceImpl_SetPrivateDataInterface(ID3D10Resource *iface, REFGUID guid,
                      const IUnknown *pData) {
    return ID3D10DeviceChildImpl_SetPrivateDataInterface((ID3D10DeviceChild *) iface, guid, pData);
}

const ID3D10ResourceVtbl D3D10Resource_Vtbl =
{
    ID3D10ResourceImpl_QueryInterface,
    ID3D10ResourceImpl_AddRef,
    ID3D10ResourceImpl_Release,
    ID3D10ResourceImpl_GetDevice,
    ID3D10ResourceImpl_GetPrivateData,
    ID3D10ResourceImpl_SetPrivateData,
    ID3D10ResourceImpl_SetPrivateDataInterface,
    ID3D10ResourceImpl_GetType,
    ID3D10ResourceImpl_SetEvictionPriority,
    ID3D10ResourceImpl_GetEvictionPriority
};
