/*
 * Direct3D 10
 *
 * Copyright 2007 Andras Kovacs
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#include "config.h"

#include "d3d10_private.h"

WINE_DEFAULT_DEBUG_CHANNEL(d3d10);

/* ID3D10Texture1D Implementation */

static HRESULT WINAPI ID3D10Texture1DImpl_QueryInterface(ID3D10Texture1D *iface, REFIID riid, void **obj) {
    ID3D10Texture1DImpl *This = (ID3D10Texture1DImpl *)iface;

    TRACE("(%p)->(%s,%p)\n", This, debugstr_guid(riid), obj);
    if(IsEqualGUID(riid, &IID_IUnknown) ||
       IsEqualGUID(riid, &IID_ID3D10Texture1D)) {
        IUnknown_AddRef(iface);
        *obj = This;
        return S_OK;
    }

    WARN("(%p)->(%s,%p),not found\n", This, debugstr_guid(riid), obj);
    *obj = NULL;
    return E_NOINTERFACE;
}

static ULONG WINAPI ID3D10Texture1DImpl_AddRef(ID3D10Texture1D *iface) {
    ID3D10Texture1DImpl *This = (ID3D10Texture1DImpl *)iface;
    ULONG ref = InterlockedIncrement(&This->resource.child.ref);

    TRACE("(%p) : AddRef from %d\n", This, ref - 1);
    return ref;
}

static ULONG WINAPI ID3D10Texture1DImpl_Release(ID3D10Texture1D *iface)
{
    ID3D10Texture1DImpl *This = (ID3D10Texture1DImpl *)iface;
    ULONG ref;

    ref = InterlockedDecrement(&This->resource.child.ref);
    TRACE("(%p) : ReleaseRef to %d\n", This, ref);

    if (ref == 0) {
        HeapFree(GetProcessHeap(), 0, This);
    }

    return ref;
}

static void WINAPI ID3D10Texture1DImpl_GetType(ID3D10Texture1D *iface, D3D10_RESOURCE_DIMENSION *rType) {
    ID3D10ResourceImpl_GetType((ID3D10Resource *) iface, rType);
}

static void WINAPI ID3D10Texture1DImpl_SetEvictionPriority(ID3D10Texture1D *iface, UINT EvictionPriority) {
    ID3D10ResourceImpl_SetEvictionPriority((ID3D10Resource *) iface, EvictionPriority);
}

static UINT WINAPI ID3D10Texture1DImpl_GetEvictionPriority(ID3D10Texture1D *iface) {
    return ID3D10ResourceImpl_GetEvictionPriority((ID3D10Resource *) iface);
}

static void WINAPI ID3D10Texture1DImpl_GetDevice(ID3D10Texture1D *iface, ID3D10Device **ppDevice) {
    ID3D10DeviceChildImpl_GetDevice((ID3D10DeviceChild *) iface, ppDevice);
}

static HRESULT WINAPI ID3D10Texture1DImpl_GetPrivateData(ID3D10Texture1D *iface, REFGUID guid,
                      UINT *pDataSize, void *pData) {
    return ID3D10DeviceChildImpl_GetPrivateData((ID3D10DeviceChild *) iface, guid, pDataSize, pData);
}

static HRESULT WINAPI ID3D10Texture1DImpl_SetPrivateData(ID3D10Texture1D *iface, REFGUID guid,
                      UINT DataSize, const void *pData) {
    return ID3D10DeviceChildImpl_SetPrivateData((ID3D10DeviceChild *) iface, guid, DataSize, pData);
}

static HRESULT WINAPI ID3D10Texture1DImpl_SetPrivateDataInterface(ID3D10Texture1D *iface, REFGUID guid,
                      const IUnknown *pData) {
    return ID3D10DeviceChildImpl_SetPrivateDataInterface((ID3D10DeviceChild *) iface, guid, pData);
}

static HRESULT WINAPI ID3D10Texture1DImpl_Map(ID3D10Texture1D *iface, UINT Subresource, D3D10_MAP MapType,
                      UINT MapFlags, void **ppData) {
    ID3D10Texture1DImpl *This = (ID3D10Texture1DImpl *)iface;

    FIXME("(This=%p, Subresource=%d, MapType=%d, MapFlags=%d, ppData=%p) stub\n",
          This, Subresource, (int) MapType, MapFlags, ppData); /* FIXME */
                      
    return S_OK;
}

static void WINAPI ID3D10Texture1DImpl_Unmap(ID3D10Texture1D *iface, UINT Subresource) {
    ID3D10Texture1DImpl *This = (ID3D10Texture1DImpl *)iface;

    FIXME("(This=%p, Subresource=%d) stub\n", This, Subresource);
}

static void WINAPI ID3D10Texture1DImpl_GetDesc(ID3D10Texture1D *iface, D3D10_TEXTURE1D_DESC *pDesc) {
    ID3D10Texture1DImpl *This = (ID3D10Texture1DImpl *)iface;

    FIXME("(This=%p, pDesc=%p) stub\n", This, pDesc);
}

const ID3D10Texture1DVtbl D3D10Texture1D_Vtbl =
{
    ID3D10Texture1DImpl_QueryInterface,
    ID3D10Texture1DImpl_AddRef,
    ID3D10Texture1DImpl_Release,
    ID3D10Texture1DImpl_GetDevice,
    ID3D10Texture1DImpl_GetPrivateData,
    ID3D10Texture1DImpl_SetPrivateData,
    ID3D10Texture1DImpl_SetPrivateDataInterface,
    ID3D10Texture1DImpl_GetType,
    ID3D10Texture1DImpl_SetEvictionPriority,
    ID3D10Texture1DImpl_GetEvictionPriority,
    ID3D10Texture1DImpl_Map,
    ID3D10Texture1DImpl_Unmap,
    ID3D10Texture1DImpl_GetDesc
};

/* ID3D10Texture2D Implementation */

static HRESULT WINAPI ID3D10Texture2DImpl_QueryInterface(ID3D10Texture2D *iface, REFIID riid, void **obj) {
    ID3D10Texture2DImpl *This = (ID3D10Texture2DImpl *)iface;

    TRACE("(%p)->(%s,%p)\n", This, debugstr_guid(riid), obj);
    if(IsEqualGUID(riid, &IID_IUnknown) ||
       IsEqualGUID(riid, &IID_ID3D10Texture2D)) {
        IUnknown_AddRef(iface);
        *obj = This;
        return S_OK;
    }

    WARN("(%p)->(%s,%p),not found\n", This, debugstr_guid(riid), obj);
    *obj = NULL;
    return E_NOINTERFACE;
}

static ULONG WINAPI ID3D10Texture2DImpl_AddRef(ID3D10Texture2D *iface) {
    ID3D10Texture2DImpl *This = (ID3D10Texture2DImpl *)iface;
    ULONG ref = InterlockedIncrement(&This->resource.child.ref);

    TRACE("(%p) : AddRef from %d\n", This, ref - 1);
    return ref;
}

static ULONG WINAPI ID3D10Texture2DImpl_Release(ID3D10Texture2D *iface) {
    ID3D10Texture2DImpl *This = (ID3D10Texture2DImpl *)iface;
    ULONG ref;

    ref = InterlockedDecrement(&This->resource.child.ref);
    TRACE("(%p) : ReleaseRef to %d\n", This, ref);

    if (ref == 0) {
        HeapFree(GetProcessHeap(), 0, This);
    }

    return ref;
}

static void WINAPI ID3D10Texture2DImpl_GetType(ID3D10Texture2D *iface, D3D10_RESOURCE_DIMENSION *rType) {
    ID3D10ResourceImpl_GetType((ID3D10Resource *) iface, rType);
}

static void WINAPI ID3D10Texture2DImpl_SetEvictionPriority(ID3D10Texture2D *iface, UINT EvictionPriority) {
    ID3D10ResourceImpl_SetEvictionPriority((ID3D10Resource *) iface, EvictionPriority);
}

static UINT WINAPI ID3D10Texture2DImpl_GetEvictionPriority(ID3D10Texture2D *iface) {
    return ID3D10ResourceImpl_GetEvictionPriority((ID3D10Resource *) iface);
}

static void WINAPI ID3D10Texture2DImpl_GetDevice(ID3D10Texture2D *iface, ID3D10Device **ppDevice) {
    ID3D10DeviceChildImpl_GetDevice((ID3D10DeviceChild *) iface, ppDevice);
}

static HRESULT WINAPI ID3D10Texture2DImpl_GetPrivateData(ID3D10Texture2D *iface, REFGUID guid,
                      UINT *pDataSize, void *pData) {
    return ID3D10DeviceChildImpl_GetPrivateData((ID3D10DeviceChild *) iface, guid, pDataSize, pData);
}

static HRESULT WINAPI ID3D10Texture2DImpl_SetPrivateData(ID3D10Texture2D *iface, REFGUID guid,
                      UINT DataSize, const void *pData) {
    return ID3D10DeviceChildImpl_SetPrivateData((ID3D10DeviceChild *) iface, guid, DataSize, pData);
}

static HRESULT WINAPI ID3D10Texture2DImpl_SetPrivateDataInterface(ID3D10Texture2D *iface, REFGUID guid,
                      const IUnknown *pData) {
    return ID3D10DeviceChildImpl_SetPrivateDataInterface((ID3D10DeviceChild *) iface, guid, pData);
}

static HRESULT WINAPI ID3D10Texture2DImpl_Map(ID3D10Texture2D *iface, UINT Subresource, D3D10_MAP MapType,
                      UINT MapFlags, D3D10_MAPPED_TEXTURE2D *pMappedTex2D) {
    ID3D10Texture2DImpl *This = (ID3D10Texture2DImpl *)iface;

    FIXME("(This=%p, Subresource=%d, MapType=%d, MapFlags=%d, pMappedTex2D=%p) stub\n",
          This, Subresource, (int) MapType, MapFlags, pMappedTex2D); /* FIXME */
                      
    return S_OK;
}

static void WINAPI ID3D10Texture2DImpl_Unmap(ID3D10Texture2D *iface, UINT Subresource) {
    ID3D10Texture2DImpl *This = (ID3D10Texture2DImpl *)iface;

    FIXME("(This=%p, Subresource=%d) stub\n", This, Subresource);
}

static void WINAPI ID3D10Texture2DImpl_GetDesc(ID3D10Texture2D *iface, D3D10_TEXTURE2D_DESC *pDesc) {
    ID3D10Texture2DImpl *This = (ID3D10Texture2DImpl *)iface;

    FIXME("(This=%p, pDesc=%p) stub\n", This, pDesc);
}

const ID3D10Texture2DVtbl D3D10Texture2D_Vtbl =
{
    ID3D10Texture2DImpl_QueryInterface,
    ID3D10Texture2DImpl_AddRef,
    ID3D10Texture2DImpl_Release,
    ID3D10Texture2DImpl_GetDevice,
    ID3D10Texture2DImpl_GetPrivateData,
    ID3D10Texture2DImpl_SetPrivateData,
    ID3D10Texture2DImpl_SetPrivateDataInterface,
    ID3D10Texture2DImpl_GetType,
    ID3D10Texture2DImpl_SetEvictionPriority,
    ID3D10Texture2DImpl_GetEvictionPriority,
    ID3D10Texture2DImpl_Map,
    ID3D10Texture2DImpl_Unmap,
    ID3D10Texture2DImpl_GetDesc
};


/* ID3D10Texture3D Implementation */

static HRESULT WINAPI ID3D10Texture3DImpl_QueryInterface(ID3D10Texture3D *iface, REFIID riid, void **obj) {
    ID3D10Texture3DImpl *This = (ID3D10Texture3DImpl *)iface;

    TRACE("(%p)->(%s,%p)\n", This, debugstr_guid(riid), obj);
    if(IsEqualGUID(riid, &IID_IUnknown) ||
       IsEqualGUID(riid, &IID_ID3D10Texture3D)) {
        IUnknown_AddRef(iface);
        *obj = This;
        return S_OK;
    }

    WARN("(%p)->(%s,%p),not found\n", This, debugstr_guid(riid), obj);
    *obj = NULL;
    return E_NOINTERFACE;
}

static ULONG WINAPI ID3D10Texture3DImpl_AddRef(ID3D10Texture3D *iface) {
    ID3D10Texture3DImpl *This = (ID3D10Texture3DImpl *)iface;
    ULONG ref = InterlockedIncrement(&This->resource.child.ref);

    TRACE("(%p) : AddRef from %d\n", This, ref - 1);
    return ref;
}

static ULONG WINAPI ID3D10Texture3DImpl_Release(ID3D10Texture3D *iface) {
    ID3D10Texture3DImpl *This = (ID3D10Texture3DImpl *)iface;
    ULONG ref;

    ref = InterlockedDecrement(&This->resource.child.ref);
    TRACE("(%p) : ReleaseRef to %d\n", This, ref);

    if (ref == 0) {
        HeapFree(GetProcessHeap(), 0, This);
    }

    return ref;
}

static void WINAPI ID3D10Texture3DImpl_GetType(ID3D10Texture3D *iface, D3D10_RESOURCE_DIMENSION *rType) {
    ID3D10ResourceImpl_GetType((ID3D10Resource *) iface, rType);
}

static void WINAPI ID3D10Texture3DImpl_SetEvictionPriority(ID3D10Texture3D *iface, UINT EvictionPriority) {
    ID3D10ResourceImpl_SetEvictionPriority((ID3D10Resource *) iface, EvictionPriority);
}

static UINT WINAPI ID3D10Texture3DImpl_GetEvictionPriority(ID3D10Texture3D *iface) {
    return ID3D10ResourceImpl_GetEvictionPriority((ID3D10Resource *) iface);
}

static void WINAPI ID3D10Texture3DImpl_GetDevice(ID3D10Texture3D *iface, ID3D10Device **ppDevice) {
    ID3D10DeviceChildImpl_GetDevice((ID3D10DeviceChild *) iface, ppDevice);
}

static HRESULT WINAPI ID3D10Texture3DImpl_GetPrivateData(ID3D10Texture3D *iface, REFGUID guid,
                      UINT *pDataSize, void *pData) {
    return ID3D10DeviceChildImpl_GetPrivateData((ID3D10DeviceChild *) iface, guid, pDataSize, pData);
}

static HRESULT WINAPI ID3D10Texture3DImpl_SetPrivateData(ID3D10Texture3D *iface, REFGUID guid,
                      UINT DataSize, const void *pData) {
    return ID3D10DeviceChildImpl_SetPrivateData((ID3D10DeviceChild *) iface, guid, DataSize, pData);
}

static HRESULT WINAPI ID3D10Texture3DImpl_SetPrivateDataInterface(ID3D10Texture3D *iface, REFGUID guid,
                      const IUnknown *pData) {
    return ID3D10DeviceChildImpl_SetPrivateDataInterface((ID3D10DeviceChild *) iface, guid, pData);
}

static HRESULT WINAPI ID3D10Texture3DImpl_Map(ID3D10Texture3D *iface, UINT Subresource, D3D10_MAP MapType,
                      UINT MapFlags, D3D10_MAPPED_TEXTURE3D *pMappedTex3D) {
    ID3D10Texture3DImpl *This = (ID3D10Texture3DImpl *)iface;

    FIXME("(This=%p, Subresource=%d, MapType=%d, MapFlags=%d, pMappedTex3D=%p) stub\n",
          This, Subresource, (int) MapType, MapFlags, pMappedTex3D); /* FIXME */
                      
    return S_OK;
}

static void WINAPI ID3D10Texture3DImpl_Unmap(ID3D10Texture3D *iface, UINT Subresource) {
    ID3D10Texture3DImpl *This = (ID3D10Texture3DImpl *)iface;

    FIXME("(This=%p, Subresource=%d) stub\n", This, Subresource);
}

static void WINAPI ID3D10Texture3DImpl_GetDesc(ID3D10Texture3D *iface, D3D10_TEXTURE3D_DESC *pDesc) {
    ID3D10Texture3DImpl *This = (ID3D10Texture3DImpl *)iface;

    FIXME("(This=%p, pDesc=%p) stub\n", This, pDesc);
}

const ID3D10Texture3DVtbl D3D10Texture3D_Vtbl =
{
    ID3D10Texture3DImpl_QueryInterface,
    ID3D10Texture3DImpl_AddRef,
    ID3D10Texture3DImpl_Release,
    ID3D10Texture3DImpl_GetDevice,
    ID3D10Texture3DImpl_GetPrivateData,
    ID3D10Texture3DImpl_SetPrivateData,
    ID3D10Texture3DImpl_SetPrivateDataInterface,
    ID3D10Texture3DImpl_GetType,
    ID3D10Texture3DImpl_SetEvictionPriority,
    ID3D10Texture3DImpl_GetEvictionPriority,
    ID3D10Texture3DImpl_Map,
    ID3D10Texture3DImpl_Unmap,
    ID3D10Texture3DImpl_GetDesc
};
