/*
 * Direct3D 10
 *
 * Copyright 2007 Andras Kovacs
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#include "config.h"

#include "d3d10_private.h"

WINE_DEFAULT_DEBUG_CHANNEL(d3d10);

const char *debug_d3d10_map(const D3D10_MAP MapType) {
    switch(MapType) {
        case D3D10_MAP_READ                 : return "D3D10_MAP_READ";
        case D3D10_MAP_WRITE                : return "D3D10_MAP_WRITE";
        case D3D10_MAP_READ_WRITE           : return "D3D10_MAP_READ_WRITE";
        case D3D10_MAP_WRITE_DISCARD        : return "D3D10_MAP_WRITE_DISCARD";
        case D3D10_MAP_WRITE_NO_OVERWRITE   : return "D3D10_MAP_WRITE_NO_OVERWRITE";
        default                             : return "unknown map type";
    }
}

const char *debug_d3d10_driver_type(const D3D10_DRIVER_TYPE DriverType) {
    switch(DriverType) {
        case D3D10_DRIVER_TYPE_HARDWARE     : return "D3D10_DRIVER_TYPE_HARDWARE";
        case D3D10_DRIVER_TYPE_REFERENCE    : return "D3D10_DRIVER_TYPE_REFERENCE";
        case D3D10_DRIVER_TYPE_NULL         : return "D3D10_DRIVER_TYPE_NULL";
        case D3D10_DRIVER_TYPE_SOFTWARE     : return "D3D10_DRIVER_TYPE_SOFTWARE";
        default                             : return "unknown driver type";
    }
}

const char *debug_dxgi_format(const DXGI_FORMAT Format) {
    switch(Format) {
        case DXGI_FORMAT_UNKNOWN                    : return "DXGI_FORMAT_UNKNOWN";
        case DXGI_FORMAT_R32G32B32A32_TYPELESS      : return "DXGI_FORMAT_R32G32B32A32_TYPELESS";
        case DXGI_FORMAT_R32G32B32A32_FLOAT         : return "DXGI_FORMAT_R32G32B32A32_FLOAT";
        case DXGI_FORMAT_R32G32B32A32_UINT          : return "DXGI_FORMAT_R32G32B32A32_UINT";
        case DXGI_FORMAT_R32G32B32A32_SINT          : return "DXGI_FORMAT_R32G32B32A32_SINT";
        case DXGI_FORMAT_R32G32B32_TYPELESS         : return "DXGI_FORMAT_R32G32B32_TYPELESS";
        case DXGI_FORMAT_R32G32B32_FLOAT            : return "DXGI_FORMAT_R32G32B32_FLOAT";
        case DXGI_FORMAT_R32G32B32_UINT             : return "DXGI_FORMAT_R32G32B32_UINT";
        case DXGI_FORMAT_R32G32B32_SINT             : return "DXGI_FORMAT_R32G32B32_SINT";
        case DXGI_FORMAT_R16G16B16A16_TYPELESS      : return "DXGI_FORMAT_R16G16B16A16_TYPELESS";
        case DXGI_FORMAT_R16G16B16A16_FLOAT         : return "DXGI_FORMAT_R16G16B16A16_FLOAT";
        case DXGI_FORMAT_R16G16B16A16_UNORM         : return "DXGI_FORMAT_R16G16B16A16_UNORM";
        case DXGI_FORMAT_R16G16B16A16_UINT          : return "DXGI_FORMAT_R16G16B16A16_UINT";
        case DXGI_FORMAT_R16G16B16A16_SNORM         : return "DXGI_FORMAT_R16G16B16A16_SNORM";
        case DXGI_FORMAT_R16G16B16A16_SINT          : return "DXGI_FORMAT_R16G16B16A16_SINT";
        case DXGI_FORMAT_R32G32_TYPELESS            : return "DXGI_FORMAT_R32G32_TYPELESS";
        case DXGI_FORMAT_R32G32_FLOAT               : return "DXGI_FORMAT_R32G32_FLOAT";
        case DXGI_FORMAT_R32G32_UINT                : return "DXGI_FORMAT_R32G32_UINT";
        case DXGI_FORMAT_R32G32_SINT                : return "DXGI_FORMAT_R32G32_SINT";
        case DXGI_FORMAT_R32G8X24_TYPELESS          : return "DXGI_FORMAT_R32G8X24_TYPELESS";
        case DXGI_FORMAT_D32_FLOAT_S8X24_UINT       : return "DXGI_FORMAT_D32_FLOAT_S8X24_UINT";
        case DXGI_FORMAT_R32_FLOAT_X8X24_TYPELESS   : return "DXGI_FORMAT_R32_FLOAT_X8X24_TYPELESS";
        case DXGI_FORMAT_X32_TYPELESS_G8X24_UINT    : return "DXGI_FORMAT_X32_TYPELESS_G8X24_UINT";
        case DXGI_FORMAT_R10G10B10A2_TYPELESS       : return "DXGI_FORMAT_R10G10B10A2_TYPELESS";
        case DXGI_FORMAT_R10G10B10A2_UNORM          : return "DXGI_FORMAT_R10G10B10A2_UNORM";
        case DXGI_FORMAT_R10G10B10A2_UINT           : return "DXGI_FORMAT_R10G10B10A2_UINT";
        case DXGI_FORMAT_R11G11B10_FLOAT            : return "DXGI_FORMAT_R11G11B10_FLOAT";
        case DXGI_FORMAT_R8G8B8A8_TYPELESS          : return "DXGI_FORMAT_R8G8B8A8_TYPELESS";
        case DXGI_FORMAT_R8G8B8A8_UNORM             : return "DXGI_FORMAT_R8G8B8A8_UNORM";
        case DXGI_FORMAT_R8G8B8A8_UNORM_SRGB        : return "DXGI_FORMAT_R8G8B8A8_UNORM_SRGB";
        case DXGI_FORMAT_R8G8B8A8_UINT              : return "DXGI_FORMAT_R8G8B8A8_UINT";
        case DXGI_FORMAT_R8G8B8A8_SNORM             : return "DXGI_FORMAT_R8G8B8A8_SNORM";
        case DXGI_FORMAT_R8G8B8A8_SINT              : return "DXGI_FORMAT_R8G8B8A8_SINT";
        case DXGI_FORMAT_R16G16_TYPELESS            : return "DXGI_FORMAT_R16G16_TYPELESS";
        case DXGI_FORMAT_R16G16_FLOAT               : return "DXGI_FORMAT_R16G16_FLOAT";
        case DXGI_FORMAT_R16G16_UNORM               : return "DXGI_FORMAT_R16G16_UNORM";
        case DXGI_FORMAT_R16G16_UINT                : return "DXGI_FORMAT_R16G16_UINT";
        case DXGI_FORMAT_R16G16_SNORM               : return "DXGI_FORMAT_R16G16_SNORM";
        case DXGI_FORMAT_R16G16_SINT                : return "DXGI_FORMAT_R16G16_SINT";
        case DXGI_FORMAT_R32_TYPELESS               : return "DXGI_FORMAT_R32_TYPELESS";
        case DXGI_FORMAT_D32_FLOAT                  : return "DXGI_FORMAT_D32_FLOAT";
        case DXGI_FORMAT_R32_FLOAT                  : return "DXGI_FORMAT_R32_FLOAT";
        case DXGI_FORMAT_R32_UINT                   : return "DXGI_FORMAT_R32_UINT";
        case DXGI_FORMAT_R32_SINT                   : return "DXGI_FORMAT_R32_SINT";
        case DXGI_FORMAT_R24G8_TYPELESS             : return "DXGI_FORMAT_R24G8_TYPELESS";
        case DXGI_FORMAT_D24_UNORM_S8_UINT          : return "DXGI_FORMAT_D24_UNORM_S8_UINT";
        case DXGI_FORMAT_R24_UNORM_X8_TYPELESS      : return "DXGI_FORMAT_R24_UNORM_X8_TYPELESS";
        case DXGI_FORMAT_X24_TYPELESS_G8_UINT       : return "DXGI_FORMAT_X24_TYPELESS_G8_UINT";
        case DXGI_FORMAT_R8G8_TYPELESS              : return "DXGI_FORMAT_R8G8_TYPELESS";
        case DXGI_FORMAT_R8G8_UNORM                 : return "DXGI_FORMAT_R8G8_UNORM";
        case DXGI_FORMAT_R8G8_UINT                  : return "DXGI_FORMAT_R8G8_UINT";
        case DXGI_FORMAT_R8G8_SNORM                 : return "DXGI_FORMAT_R8G8_SNORM";
        case DXGI_FORMAT_R8G8_SINT                  : return "DXGI_FORMAT_R8G8_SINT";
        case DXGI_FORMAT_R16_TYPELESS               : return "DXGI_FORMAT_R16_TYPELESS";
        case DXGI_FORMAT_R16_FLOAT                  : return "DXGI_FORMAT_R16_FLOAT";
        case DXGI_FORMAT_D16_UNORM                  : return "DXGI_FORMAT_D16_UNORM";
        case DXGI_FORMAT_R16_UNORM                  : return "DXGI_FORMAT_R16_UNORM";
        case DXGI_FORMAT_R16_UINT                   : return "DXGI_FORMAT_R16_UINT";
        case DXGI_FORMAT_R16_SNORM                  : return "DXGI_FORMAT_R16_SNORM";
        case DXGI_FORMAT_R16_SINT                   : return "DXGI_FORMAT_R16_SINT";
        case DXGI_FORMAT_R8_TYPELESS                : return "DXGI_FORMAT_R8_TYPELESS";
        case DXGI_FORMAT_R8_UNORM                   : return "DXGI_FORMAT_R8_UNORM";
        case DXGI_FORMAT_R8_UINT                    : return "DXGI_FORMAT_R8_UINT";
        case DXGI_FORMAT_R8_SNORM                   : return "DXGI_FORMAT_R8_SNORM";
        case DXGI_FORMAT_R8_SINT                    : return "DXGI_FORMAT_R8_SINT";
        case DXGI_FORMAT_A8_UNORM                   : return "DXGI_FORMAT_A8_UNORM";
        case DXGI_FORMAT_R1_UNORM                   : return "DXGI_FORMAT_R1_UNORM";
        case DXGI_FORMAT_R9G9B9E5_SHAREDEXP         : return "DXGI_FORMAT_R9G9B9E5_SHAREDEXP";
        case DXGI_FORMAT_R8G8_B8G8_UNORM            : return "DXGI_FORMAT_R8G8_B8G8_UNORM";
        case DXGI_FORMAT_G8R8_G8B8_UNORM            : return "DXGI_FORMAT_G8R8_G8B8_UNORM";
        case DXGI_FORMAT_BC1_TYPELESS               : return "DXGI_FORMAT_BC1_TYPELESS";
        case DXGI_FORMAT_BC1_UNORM                  : return "DXGI_FORMAT_BC1_UNORM";
        case DXGI_FORMAT_BC1_UNORM_SRGB             : return "DXGI_FORMAT_BC1_UNORM_SRGB";
        case DXGI_FORMAT_BC2_TYPELESS               : return "DXGI_FORMAT_BC2_TYPELESS";
        case DXGI_FORMAT_BC2_UNORM                  : return "DXGI_FORMAT_BC2_UNORM";
        case DXGI_FORMAT_BC2_UNORM_SRGB             : return "DXGI_FORMAT_BC2_UNORM_SRGB";
        case DXGI_FORMAT_BC3_TYPELESS               : return "DXGI_FORMAT_BC3_TYPELESS";
        case DXGI_FORMAT_BC3_UNORM                  : return "DXGI_FORMAT_BC3_UNORM";
        case DXGI_FORMAT_BC3_UNORM_SRGB             : return "DXGI_FORMAT_BC3_UNORM_SRGB";
        case DXGI_FORMAT_BC4_TYPELESS               : return "DXGI_FORMAT_BC4_TYPELESS";
        case DXGI_FORMAT_BC4_UNORM                  : return "DXGI_FORMAT_BC4_UNORM";
        case DXGI_FORMAT_BC4_SNORM                  : return "DXGI_FORMAT_BC4_SNORM";
        case DXGI_FORMAT_BC5_TYPELESS               : return "DXGI_FORMAT_BC5_TYPELESS";
        case DXGI_FORMAT_BC5_UNORM                  : return "DXGI_FORMAT_BC5_UNORM";
        case DXGI_FORMAT_BC5_SNORM                  : return "DXGI_FORMAT_BC5_SNORM";
        case DXGI_FORMAT_B5G6R5_UNORM               : return "DXGI_FORMAT_B5G6R5_UNORM";
        case DXGI_FORMAT_B5G5R5A1_UNORM             : return "DXGI_FORMAT_B5G5R5A1_UNORM";
        case DXGI_FORMAT_B8G8R8A8_UNORM             : return "DXGI_FORMAT_B8G8R8A8_UNORM";
        case DXGI_FORMAT_B8G8R8X8_UNORM             : return "DXGI_FORMAT_B8G8R8X8_UNORM";
        default                                     : return "unexpected pixelformat";
    }
}

const char *debug_d3d10_blend(const D3D10_BLEND Blend) {
    switch(Blend) {
        case D3D10_BLEND_ZERO               : return "D3D10_BLEND_ZERO";
        case D3D10_BLEND_ONE                : return "D3D10_BLEND_ONE";
        case D3D10_BLEND_SRC_COLOR          : return "D3D10_BLEND_SRC_COLOR";
        case D3D10_BLEND_INV_SRC_COLOR      : return "D3D10_BLEND_INV_SRC_COLOR";
        case D3D10_BLEND_SRC_ALPHA          : return "D3D10_BLEND_SRC_ALPHA";
        case D3D10_BLEND_INV_SRC_ALPHA      : return "D3D10_BLEND_INV_SRC_ALPHA";
        case D3D10_BLEND_DEST_ALPHA         : return "D3D10_BLEND_DEST_ALPHA";
        case D3D10_BLEND_INV_DEST_ALPHA     : return "D3D10_BLEND_INV_DEST_ALPHA";
        case D3D10_BLEND_DEST_COLOR         : return "D3D10_BLEND_DEST_COLOR";
        case D3D10_BLEND_INV_DEST_COLOR     : return "D3D10_BLEND_INV_DEST_COLOR";
        case D3D10_BLEND_SRC_ALPHA_SAT      : return "D3D10_BLEND_SRC_ALPHA_SAT";
        case D3D10_BLEND_BLEND_FACTOR       : return "D3D10_BLEND_BLEND_FACTOR";
        case D3D10_BLEND_INV_BLEND_FACTOR   : return "D3D10_BLEND_INV_BLEND_FACTOR";
        case D3D10_BLEND_SRC1_COLOR         : return "D3D10_BLEND_SRC1_COLOR";
        case D3D10_BLEND_INV_SRC1_COLOR     : return "D3D10_BLEND_INV_SRC1_COLOR";
        case D3D10_BLEND_SRC1_ALPHA         : return "D3D10_BLEND_SRC1_ALPHA";
        case D3D10_BLEND_INV_SRC1_ALPHA     : return "D3D10_BLEND_INV_SRC1_ALPHA";
        default                             : return "unknown d3d10 blend value";
    }
}
const char *debug_d3d10_blend_op(const D3D10_BLEND_OP BlendOp) {
    switch(BlendOp) {
        case D3D10_BLEND_OP_ADD             : return "D3D10_BLEND_OP_ADD";
        case D3D10_BLEND_OP_SUBTRACT        : return "D3D10_BLEND_OP_SUBTRACT";
        case D3D10_BLEND_OP_REV_SUBTRACT    : return "D3D10_BLEND_OP_REV_SUBTRACT";
        case D3D10_BLEND_OP_MIN             : return "D3D10_BLEND_OP_MIN";
        case D3D10_BLEND_OP_MAX             : return "D3D10_BLEND_OP_MAX";
        default                             : return "unknown d3d10 blend op";
    }
}

void dump_d3d10_blend_desc(const D3D10_BLEND_DESC *pBlendStateDesc) {
    TRACE("Dumping D3D10_BLEND_DESC structure:\n");
    TRACE("AlphaToCoverageEnable    = %s\n", pBlendStateDesc->AlphaToCoverageEnable ? "TRUE" : "FALSE");
    TRACE("BlendEnable[0]           = %s\n", pBlendStateDesc->BlendEnable[0] ? "TRUE" : "FALSE");
    TRACE("BlendEnable[1]           = %s\n", pBlendStateDesc->BlendEnable[1] ? "TRUE" : "FALSE");
    TRACE("BlendEnable[2]           = %s\n", pBlendStateDesc->BlendEnable[2] ? "TRUE" : "FALSE");
    TRACE("BlendEnable[3]           = %s\n", pBlendStateDesc->BlendEnable[3] ? "TRUE" : "FALSE");
    TRACE("BlendEnable[4]           = %s\n", pBlendStateDesc->BlendEnable[4] ? "TRUE" : "FALSE");
    TRACE("BlendEnable[5]           = %s\n", pBlendStateDesc->BlendEnable[5] ? "TRUE" : "FALSE");
    TRACE("BlendEnable[6]           = %s\n", pBlendStateDesc->BlendEnable[6] ? "TRUE" : "FALSE");
    TRACE("BlendEnable[7]           = %s\n", pBlendStateDesc->BlendEnable[7] ? "TRUE" : "FALSE");
    TRACE("SrcBlend                 = %s\n", debug_d3d10_blend(pBlendStateDesc->SrcBlend));
    TRACE("DestBlend                = %s\n", debug_d3d10_blend(pBlendStateDesc->DestBlend));
    TRACE("BlendOp                  = %s\n", debug_d3d10_blend_op(pBlendStateDesc->BlendOp));
    TRACE("SrcBlendAlpha            = %s\n", debug_d3d10_blend(pBlendStateDesc->SrcBlendAlpha));
    TRACE("DestBlendAlpha           = %s\n", debug_d3d10_blend(pBlendStateDesc->DestBlendAlpha));
    TRACE("BlendOpAlpha             = %s\n", debug_d3d10_blend_op(pBlendStateDesc->BlendOpAlpha));
    TRACE("RenderTargetWriteMask[0] = %08xd\n", pBlendStateDesc->RenderTargetWriteMask[0]);
    TRACE("RenderTargetWriteMask[1] = %08xd\n", pBlendStateDesc->RenderTargetWriteMask[1]);
    TRACE("RenderTargetWriteMask[2] = %08xd\n", pBlendStateDesc->RenderTargetWriteMask[2]);
    TRACE("RenderTargetWriteMask[3] = %08xd\n", pBlendStateDesc->RenderTargetWriteMask[3]);
    TRACE("RenderTargetWriteMask[4] = %08xd\n", pBlendStateDesc->RenderTargetWriteMask[4]);
    TRACE("RenderTargetWriteMask[5] = %08xd\n", pBlendStateDesc->RenderTargetWriteMask[5]);
    TRACE("RenderTargetWriteMask[6] = %08xd\n", pBlendStateDesc->RenderTargetWriteMask[6]);
    TRACE("RenderTargetWriteMask[7] = %08xd\n", pBlendStateDesc->RenderTargetWriteMask[7]);
}

const char *debug_d3d10_depth_write_mask(const D3D10_DEPTH_WRITE_MASK Mask) {
    switch(Mask) {
        case D3D10_DEPTH_WRITE_MASK_ZERO    : return "D3D10_DEPTH_WRITE_MASK_ZERO";
        case D3D10_DEPTH_WRITE_MASK_ALL     : return "D3D10_DEPTH_WRITE_MASK_ALL";
        default                             : return "unknown depth write mask";
    }
}

const char *debug_d3d10_comparison_func(const D3D10_COMPARISON_FUNC Func) {
    switch(Func) {
        case D3D10_COMPARISON_NEVER         : return "D3D10_COMPARISON_NEVER";
        case D3D10_COMPARISON_LESS          : return "D3D10_COMPARISON_LESS";
        case D3D10_COMPARISON_EQUAL         : return "D3D10_COMPARISON_EQUAL";
        case D3D10_COMPARISON_LESS_EQUAL    : return "D3D10_COMPARISON_LESS_EQUAL";
        case D3D10_COMPARISON_GREATER       : return "D3D10_COMPARISON_GREATER";
        case D3D10_COMPARISON_NOT_EQUAL     : return "D3D10_COMPARISON_NOT_EQUAL";
        case D3D10_COMPARISON_GREATER_EQUAL : return "D3D10_COMPARISON_GREATER_EQUAL";
        case D3D10_COMPARISON_ALWAYS        : return "D3D10_COMPARISON_ALWAYS";
        default                             : return "unexpected comparison function";
    }
}

const char *debug_d3d10_stencil_op(const D3D10_STENCIL_OP op) {
    switch(op) {
        case D3D10_STENCIL_OP_KEEP          : return "D3D10_STENCIL_OP_KEEP";
        case D3D10_STENCIL_OP_ZERO          : return "D3D10_STENCIL_OP_ZERO";
        case D3D10_STENCIL_OP_REPLACE       : return "D3D10_STENCIL_OP_REPLACE";
        case D3D10_STENCIL_OP_INCR_SAT      : return "D3D10_STENCIL_OP_INCR_SAT";
        case D3D10_STENCIL_OP_DECR_SAT      : return "D3D10_STENCIL_OP_DECR_SAT";
        case D3D10_STENCIL_OP_INVERT        : return "D3D10_STENCIL_OP_INVERT";
        case D3D10_STENCIL_OP_INCR          : return "D3D10_STENCIL_OP_INCR";
        case D3D10_STENCIL_OP_DECR          : return "D3D10_STENCIL_OP_DECR";
        default                             : return "unexpected stencil operation";
    }
}

void dump_d3d10_depth_stencilop_desc(const D3D10_DEPTH_STENCILOP_DESC Func) {
    TRACE("StencilFailOp            = %s\n", debug_d3d10_stencil_op(Func.StencilFailOp));
    TRACE("StencilDepthFailOp       = %s\n", debug_d3d10_stencil_op(Func.StencilDepthFailOp));
    TRACE("StencilPassOp            = %s\n", debug_d3d10_stencil_op(Func.StencilPassOp));
}

void dump_d3d10_depth_stencil_desc(const D3D10_DEPTH_STENCIL_DESC *pDepthStencilDesc) {
    TRACE("DepthEnable              = %s\n", pDepthStencilDesc->DepthEnable ? "TRUE" : "FALSE");
    TRACE("DepthWriteMask           = %s\n", debug_d3d10_depth_write_mask(pDepthStencilDesc->DepthWriteMask));
    TRACE("DepthFunc                = %s\n", debug_d3d10_comparison_func(pDepthStencilDesc->DepthFunc));
    TRACE("StencilEnable            = %s\n", pDepthStencilDesc->StencilEnable ? "TRUE" : "FALSE");
    TRACE("StencilReadMask          = %08x\n", pDepthStencilDesc->StencilReadMask);
    TRACE("StencilWriteMask         = %08x\n", pDepthStencilDesc->StencilWriteMask);
    TRACE("Front face:\n");
    dump_d3d10_depth_stencilop_desc(pDepthStencilDesc->FrontFace);
    TRACE("Back face:\n");
    dump_d3d10_depth_stencilop_desc(pDepthStencilDesc->BackFace);
}

const char *debug_d3d10_clear_flags(const UINT ClearFlags) {
    if(ClearFlags == (D3D10_CLEAR_DEPTH | D3D10_CLEAR_STENCIL)) {
        return "D3D10_CLEAR_DEPTH | D3D10_CLEAR_STENCIL";
    } else if(ClearFlags == D3D10_CLEAR_DEPTH) {
        return "D3D10_CLEAR_DEPTH";
    } else if( ClearFlags == D3D10_CLEAR_STENCIL) {
        return "D3D10_CLEAR_STENCIL";
    } else {
        return "unexpected clear flag combination";
    }
}

const char *debug_d3d10_usage(const D3D10_USAGE Usage) {
    switch (Usage) {
    case D3D10_USAGE_DEFAULT		: return "D3D10_USAGE_DEFAULT";
    case D3D10_USAGE_IMMUTABLE		: return "D3D10_USAGE_IMMUTABLE";
    case D3D10_USAGE_DYNAMIC		: return "D3D10_USAGE_DYNAMIC";
    case D3D10_USAGE_STAGING		: return "D3D10_USAGE_STAGING";
    default				: return "unexpected usage";
    }
}

void dump_d3d10_buffer_desc(D3D10_BUFFER_DESC *pDesc) {
    TRACE("ByteWidth		= %d\n", pDesc->ByteWidth);
    TRACE("Usage		= %d (%s)\n", (int) pDesc->Usage, debug_d3d10_usage(pDesc->Usage));
    TRACE("BindFlags		= %d\n", pDesc->BindFlags);
    TRACE("CPUAccessFlags	= %d\n", pDesc->CPUAccessFlags);
    TRACE("MiscFlags		= %d\n", pDesc->MiscFlags);
}

const char *debug_d3d10_counter(const D3D10_COUNTER cnt) {
    switch (cnt) {
    case D3D10_COUNTER_GPU_IDLE					: return "D3D10_COUNTER_GPU_IDLE";
    case D3D10_COUNTER_VERTEX_PROCESSING			: return "D3D10_COUNTER_VERTEX_PROCESSING";
    case D3D10_COUNTER_GEOMETRY_PROCESSING			: return "D3D10_COUNTER_GEOMETRY_PROCESSING";
    case D3D10_COUNTER_PIXEL_PROCESSING				: return "D3D10_COUNTER_PIXEL_PROCESSING";
    case D3D10_COUNTER_OTHER_GPU_PROCESSING			: return "D3D10_COUNTER_OTHER_GPU_PROCESSING";
    case D3D10_COUNTER_HOST_ADAPTER_BANDWIDTH_UTILIZATION	: return "D3D10_COUNTER_HOST_ADAPTER_BANDWIDTH_UTILIZATION";
    case D3D10_COUNTER_LOCAL_VIDMEM_BANDWIDTH_UTILIZATION	: return "D3D10_COUNTER_LOCAL_VIDMEM_BANDWIDTH_UTILIZATION";
    case D3D10_COUNTER_VERTEX_THROUGHPUT_UTILIZATION		: return "D3D10_COUNTER_VERTEX_THROUGHPUT_UTILIZATION";
    case D3D10_COUNTER_TRIANGLE_SETUP_THROUGHPUT_UTILIZATION	: return "D3D10_COUNTER_TRIANGLE_SETUP_THROUGHPUT_UTILIZATION";
    case D3D10_COUNTER_FILLRATE_THROUGHPUT_UTILIZATION		: return "D3D10_COUNTER_FILLRATE_THROUGHPUT_UTILIZATION";
    case D3D10_COUNTER_VS_MEMORY_LIMITED			: return "D3D10_COUNTER_VS_MEMORY_LIMITED";
    case D3D10_COUNTER_VS_COMPUTATION_LIMITED			: return "D3D10_COUNTER_VS_COMPUTATION_LIMITED";
    case D3D10_COUNTER_GS_MEMORY_LIMITED			: return "D3D10_COUNTER_GS_MEMORY_LIMITED";
    case D3D10_COUNTER_GS_COMPUTATION_LIMITED			: return "D3D10_COUNTER_GS_COMPUTATION_LIMITED";
    case D3D10_COUNTER_PS_MEMORY_LIMITED			: return "D3D10_COUNTER_PS_MEMORY_LIMITED";
    case D3D10_COUNTER_PS_COMPUTATION_LIMITED			: return "D3D10_COUNTER_PS_COMPUTATION_LIMITED";
    case D3D10_COUNTER_POST_TRANSFORM_CACHE_HIT_RATE		: return "D3D10_COUNTER_POST_TRANSFORM_CACHE_HIT_RATE";
    case D3D10_COUNTER_TEXTURE_CACHE_HIT_RATE			: return "D3D10_COUNTER_TEXTURE_CACHE_HIT_RATE";
    case D3D10_COUNTER_DEVICE_DEPENDENT_0			: return "D3D10_COUNTER_DEVICE_DEPENDENT_0";
    default 							: return "unexpected counter";
    }
}

void dump_d3d10_counter_desc (D3D10_COUNTER_DESC *pDesc) {
    TRACE("Counter		= %d (%s)\n", (int) pDesc->Counter, debug_d3d10_counter(pDesc->Counter));
    TRACE("MiscFlags		= %d\n",  pDesc->MiscFlags);
}
