/*
 * Direct3D 10
 *
 * Copyright 2007 Andras Kovacs
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 *
 */

#include "config.h"

#include "d3d10_private.h"

WINE_DEFAULT_DEBUG_CHANNEL(d3d10);

/* ID3D10View Implementation */
static HRESULT WINAPI ID3D10ViewImpl_QueryInterface(ID3D10View *iface, REFIID riid, void **obj) {
    ID3D10ViewImpl *This = (ID3D10ViewImpl *)iface;

    TRACE("(%p)->(%s,%p)\n", This, debugstr_guid(riid), obj);
    if(IsEqualGUID(riid, &IID_IUnknown) ||
       IsEqualGUID(riid, &IID_ID3D10View)) {
        IUnknown_AddRef(iface);
        *obj = This;
        return S_OK;
    }

    WARN("(%p)->(%s,%p),not found\n", This, debugstr_guid(riid), obj);
    *obj = NULL;
    return E_NOINTERFACE;
}

static ULONG WINAPI ID3D10ViewImpl_AddRef(ID3D10View *iface) {
    ID3D10ViewImpl *This = (ID3D10ViewImpl *)iface;
    ULONG ref = InterlockedIncrement(&This->view.child.ref);

    TRACE("(%p) : AddRef from %d\n", This, ref - 1);
    return ref;
}

static ULONG WINAPI ID3D10ViewImpl_Release(ID3D10View *iface) {
    ID3D10ViewImpl *This = (ID3D10ViewImpl *)iface;
    ULONG ref;

    ref = InterlockedDecrement(&This->view.child.ref);
    TRACE("(%p) : ReleaseRef to %d\n", This, ref);

    if (ref == 0) {
        HeapFree(GetProcessHeap(), 0, This);
    }

    return ref;
}

void WINAPI ID3D10ViewImpl_GetResource(ID3D10View *iface, ID3D10Resource **ppResource) {
    ID3D10ViewImpl *This = (ID3D10ViewImpl *)iface;
    FIXME("(This=%p, ppResource=%p) stub\n", This, ppResource);
}


static void WINAPI ID3D10ViewImpl_GetDevice(ID3D10View *iface, ID3D10Device **ppDevice) {
    ID3D10DeviceChildImpl_GetDevice((ID3D10DeviceChild *) iface, ppDevice);
}

static HRESULT WINAPI ID3D10ViewImpl_GetPrivateData(ID3D10View *iface, REFGUID guid,
                      UINT *pDataSize, void *pData) {
    return ID3D10DeviceChildImpl_GetPrivateData((ID3D10DeviceChild *) iface, guid, pDataSize, pData);
}

static HRESULT WINAPI ID3D10ViewImpl_SetPrivateData(ID3D10View *iface, REFGUID guid,
                      UINT DataSize, const void *pData) {
    return ID3D10DeviceChildImpl_SetPrivateData((ID3D10DeviceChild *) iface, guid, DataSize, pData);
}

static HRESULT WINAPI ID3D10ViewImpl_SetPrivateDataInterface(ID3D10View *iface, REFGUID guid,
                      const IUnknown *pData) {
    return ID3D10DeviceChildImpl_SetPrivateDataInterface((ID3D10DeviceChild *) iface, guid, pData);
}

const ID3D10ViewVtbl D3D10View_Vtbl =
{
    ID3D10ViewImpl_QueryInterface,
    ID3D10ViewImpl_AddRef,
    ID3D10ViewImpl_Release,
    ID3D10ViewImpl_GetDevice,
    ID3D10ViewImpl_GetPrivateData,
    ID3D10ViewImpl_SetPrivateData,
    ID3D10ViewImpl_SetPrivateDataInterface,
    ID3D10ViewImpl_GetResource
};

/* ID3D10DepthStencilView Implementation */
static HRESULT WINAPI ID3D10DepthStencilViewImpl_QueryInterface(ID3D10DepthStencilView *iface, REFIID riid,
               void **obj) {
    ID3D10DepthStencilViewImpl *This = (ID3D10DepthStencilViewImpl *)iface;

    TRACE("(%p)->(%s,%p)\n", This, debugstr_guid(riid), obj);
    if(IsEqualGUID(riid, &IID_IUnknown) ||
       IsEqualGUID(riid, &IID_ID3D10DepthStencilView)) {
        IUnknown_AddRef(iface);
        *obj = This;
        return S_OK;
    }

    WARN("(%p)->(%s,%p),not found\n", This, debugstr_guid(riid), obj);
    *obj = NULL;
    return E_NOINTERFACE;
}

static ULONG WINAPI ID3D10DepthStencilViewImpl_AddRef(ID3D10DepthStencilView *iface) {
    ID3D10DepthStencilViewImpl *This = (ID3D10DepthStencilViewImpl *)iface;
    ULONG ref = InterlockedIncrement(&This->view.child.ref);

    TRACE("(%p) : AddRef from %d\n", This, ref - 1);
    return ref;
}

static ULONG WINAPI ID3D10DepthStencilViewImpl_Release(ID3D10DepthStencilView *iface) {
    ID3D10DepthStencilViewImpl *This = (ID3D10DepthStencilViewImpl *)iface;
    ULONG ref;

    ref = InterlockedDecrement(&This->view.child.ref);
    TRACE("(%p) : ReleaseRef to %d\n", This, ref);

    if (ref == 0) {
        HeapFree(GetProcessHeap(), 0, This);
    }

    return ref;
}

static void WINAPI ID3D10DepthStencilViewImpl_GetDesc(ID3D10DepthStencilView *iface,
                   D3D10_DEPTH_STENCIL_VIEW_DESC *pDesc) {
    ID3D10DepthStencilViewImpl *This = (ID3D10DepthStencilViewImpl *)iface;

    FIXME("(This=%p, pDesc=%p) stub\n", This, pDesc);    
}

static void WINAPI ID3D10DepthStencilViewImpl_GetResource(ID3D10DepthStencilView *iface, 
                   ID3D10Resource **ppResource) {
    ID3D10ViewImpl_GetResource((ID3D10View *) iface, ppResource);
}


static void WINAPI ID3D10DepthStencilViewImpl_GetDevice(ID3D10DepthStencilView *iface, ID3D10Device **ppDevice) {
    ID3D10DeviceChildImpl_GetDevice((ID3D10DeviceChild *) iface, ppDevice);
}

static HRESULT WINAPI ID3D10DepthStencilViewImpl_GetPrivateData(ID3D10DepthStencilView *iface, REFGUID guid,
                      UINT *pDataSize, void *pData) {
    return ID3D10DeviceChildImpl_GetPrivateData((ID3D10DeviceChild *) iface, guid, pDataSize, pData);
}

static HRESULT WINAPI ID3D10DepthStencilViewImpl_SetPrivateData(ID3D10DepthStencilView *iface, REFGUID guid,
                      UINT DataSize, const void *pData) {
    return ID3D10DeviceChildImpl_SetPrivateData((ID3D10DeviceChild *) iface, guid, DataSize, pData);
}

static HRESULT WINAPI ID3D10DepthStencilViewImpl_SetPrivateDataInterface(ID3D10DepthStencilView *iface, REFGUID guid,
                      const IUnknown *pData) {
    return ID3D10DeviceChildImpl_SetPrivateDataInterface((ID3D10DeviceChild *) iface, guid, pData);
}

const ID3D10DepthStencilViewVtbl D3D10DepthStencilView_Vtbl =
{
    ID3D10DepthStencilViewImpl_QueryInterface,
    ID3D10DepthStencilViewImpl_AddRef,
    ID3D10DepthStencilViewImpl_Release,
    ID3D10DepthStencilViewImpl_GetDevice,
    ID3D10DepthStencilViewImpl_GetPrivateData,
    ID3D10DepthStencilViewImpl_SetPrivateData,
    ID3D10DepthStencilViewImpl_SetPrivateDataInterface,
    ID3D10DepthStencilViewImpl_GetResource,
    ID3D10DepthStencilViewImpl_GetDesc
};

/* ID3D10RenderTargetView Implementation */
static HRESULT WINAPI ID3D10RenderTargetViewImpl_QueryInterface(ID3D10RenderTargetView *iface, REFIID riid,
               void **obj) {
    ID3D10RenderTargetViewImpl *This = (ID3D10RenderTargetViewImpl *)iface;

    TRACE("(%p)->(%s,%p)\n", This, debugstr_guid(riid), obj);
    if(IsEqualGUID(riid, &IID_IUnknown) ||
       IsEqualGUID(riid, &IID_ID3D10RenderTargetView)) {
        IUnknown_AddRef(iface);
        *obj = This;
        return S_OK;
    }

    WARN("(%p)->(%s,%p),not found\n", This, debugstr_guid(riid), obj);
    *obj = NULL;
    return E_NOINTERFACE;
}

static ULONG WINAPI ID3D10RenderTargetViewImpl_AddRef(ID3D10RenderTargetView *iface) {
    ID3D10RenderTargetViewImpl *This = (ID3D10RenderTargetViewImpl *)iface;
    ULONG ref = InterlockedIncrement(&This->view.child.ref);

    TRACE("(%p) : AddRef from %d\n", This, ref - 1);
    return ref;
}

static ULONG WINAPI ID3D10RenderTargetViewImpl_Release(ID3D10RenderTargetView *iface) {
    ID3D10RenderTargetViewImpl *This = (ID3D10RenderTargetViewImpl *)iface;
    ULONG ref;

    ref = InterlockedDecrement(&This->view.child.ref);
    TRACE("(%p) : ReleaseRef to %d\n", This, ref);

    if (ref == 0) {
        HeapFree(GetProcessHeap(), 0, This);
    }

    return ref;
}

static void WINAPI ID3D10RenderTargetViewImpl_GetDesc(ID3D10RenderTargetView *iface,
                   D3D10_RENDER_TARGET_VIEW_DESC *pDesc) {
    ID3D10RenderTargetViewImpl *This = (ID3D10RenderTargetViewImpl *)iface;

    FIXME("(This=%p, pDesc=%p) stub\n", This, pDesc);    
}

static void WINAPI ID3D10RenderTargetViewImpl_GetResource(ID3D10RenderTargetView *iface, 
                   ID3D10Resource **ppResource) {
    ID3D10ViewImpl_GetResource((ID3D10View *) iface, ppResource);
}


static void WINAPI ID3D10RenderTargetViewImpl_GetDevice(ID3D10RenderTargetView *iface, ID3D10Device **ppDevice) {
    ID3D10DeviceChildImpl_GetDevice((ID3D10DeviceChild *) iface, ppDevice);
}

static HRESULT WINAPI ID3D10RenderTargetViewImpl_GetPrivateData(ID3D10RenderTargetView *iface, REFGUID guid,
                      UINT *pDataSize, void *pData) {
    return ID3D10DeviceChildImpl_GetPrivateData((ID3D10DeviceChild *) iface, guid, pDataSize, pData);
}

static HRESULT WINAPI ID3D10RenderTargetViewImpl_SetPrivateData(ID3D10RenderTargetView *iface, REFGUID guid,
                      UINT DataSize, const void *pData) {
    return ID3D10DeviceChildImpl_SetPrivateData((ID3D10DeviceChild *) iface, guid, DataSize, pData);
}

static HRESULT WINAPI ID3D10RenderTargetViewImpl_SetPrivateDataInterface(ID3D10RenderTargetView *iface, REFGUID guid,
                      const IUnknown *pData) {
    return ID3D10DeviceChildImpl_SetPrivateDataInterface((ID3D10DeviceChild *) iface, guid, pData);
}

const ID3D10RenderTargetViewVtbl D3D10RenderTargetView_Vtbl =
{
    ID3D10RenderTargetViewImpl_QueryInterface,
    ID3D10RenderTargetViewImpl_AddRef,
    ID3D10RenderTargetViewImpl_Release,
    ID3D10RenderTargetViewImpl_GetDevice,
    ID3D10RenderTargetViewImpl_GetPrivateData,
    ID3D10RenderTargetViewImpl_SetPrivateData,
    ID3D10RenderTargetViewImpl_SetPrivateDataInterface,
    ID3D10RenderTargetViewImpl_GetResource,
    ID3D10RenderTargetViewImpl_GetDesc
};

/* ID3D10ShaderResourceView Implementation */
static HRESULT WINAPI ID3D10ShaderResourceViewImpl_QueryInterface(ID3D10ShaderResourceView *iface, REFIID riid,
               void **obj) {
    ID3D10ShaderResourceViewImpl *This = (ID3D10ShaderResourceViewImpl *)iface;

    TRACE("(%p)->(%s,%p)\n", This, debugstr_guid(riid), obj);
    if(IsEqualGUID(riid, &IID_IUnknown) ||
       IsEqualGUID(riid, &IID_ID3D10ShaderResourceView)) {
        IUnknown_AddRef(iface);
        *obj = This;
        return S_OK;
    }

    WARN("(%p)->(%s,%p),not found\n", This, debugstr_guid(riid), obj);
    *obj = NULL;
    return E_NOINTERFACE;
}

static ULONG WINAPI ID3D10ShaderResourceViewImpl_AddRef(ID3D10ShaderResourceView *iface) {
    ID3D10ShaderResourceViewImpl *This = (ID3D10ShaderResourceViewImpl *)iface;
    ULONG ref = InterlockedIncrement(&This->child.ref);

    TRACE("(%p) : AddRef from %d\n", This, ref - 1);
    return ref;
}

static ULONG WINAPI ID3D10ShaderResourceViewImpl_Release(ID3D10ShaderResourceView *iface) {
    ID3D10ShaderResourceViewImpl *This = (ID3D10ShaderResourceViewImpl *)iface;
    ULONG ref;

    ref = InterlockedDecrement(&This->child.ref);
    TRACE("(%p) : ReleaseRef to %d\n", This, ref);

    if (ref == 0) {
        HeapFree(GetProcessHeap(), 0, This);
    }

    return ref;
}

static void WINAPI ID3D10ShaderResourceViewImpl_GetDesc(ID3D10ShaderResourceView *iface,
                   D3D10_SHADER_RESOURCE_VIEW_DESC *pDesc) {
    ID3D10ShaderResourceViewImpl *This = (ID3D10ShaderResourceViewImpl *)iface;

    FIXME("(This=%p, pDesc=%p) stub\n", This, pDesc);    
}

static void WINAPI ID3D10ShaderResourceViewImpl_GetResource(ID3D10ShaderResourceView *iface, 
                   ID3D10Resource **ppResource) {
    ID3D10ViewImpl_GetResource((ID3D10View *) iface, ppResource);
}


static void WINAPI ID3D10ShaderResourceViewImpl_GetDevice(ID3D10ShaderResourceView *iface, ID3D10Device **ppDevice) {
    ID3D10DeviceChildImpl_GetDevice((ID3D10DeviceChild *) iface, ppDevice);
}

static HRESULT WINAPI ID3D10ShaderResourceViewImpl_GetPrivateData(ID3D10ShaderResourceView *iface, REFGUID guid,
                      UINT *pDataSize, void *pData) {
    return ID3D10DeviceChildImpl_GetPrivateData((ID3D10DeviceChild *) iface, guid, pDataSize, pData);
}

static HRESULT WINAPI ID3D10ShaderResourceViewImpl_SetPrivateData(ID3D10ShaderResourceView *iface, REFGUID guid,
                      UINT DataSize, const void *pData) {
    return ID3D10DeviceChildImpl_SetPrivateData((ID3D10DeviceChild *) iface, guid, DataSize, pData);
}

static HRESULT WINAPI ID3D10ShaderResourceViewImpl_SetPrivateDataInterface(ID3D10ShaderResourceView *iface, REFGUID guid,
                      const IUnknown *pData) {
    return ID3D10DeviceChildImpl_SetPrivateDataInterface((ID3D10DeviceChild *) iface, guid, pData);
}

const ID3D10ShaderResourceViewVtbl D3D10ShaderResourceView_Vtbl =
{
    ID3D10ShaderResourceViewImpl_QueryInterface,
    ID3D10ShaderResourceViewImpl_AddRef,
    ID3D10ShaderResourceViewImpl_Release,
    ID3D10ShaderResourceViewImpl_GetDevice,
    ID3D10ShaderResourceViewImpl_GetPrivateData,
    ID3D10ShaderResourceViewImpl_SetPrivateData,
    ID3D10ShaderResourceViewImpl_SetPrivateDataInterface,
    ID3D10ShaderResourceViewImpl_GetResource,
    ID3D10ShaderResourceViewImpl_GetDesc
};
